"""Generated client library for baremetalsolution version v2."""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.py import base_api
from googlecloudsdk.generated_clients.apis.baremetalsolution.v2 import baremetalsolution_v2_messages as messages


class BaremetalsolutionV2(base_api.BaseApiClient):
  """Generated client library for service baremetalsolution version v2."""

  MESSAGES_MODULE = messages
  BASE_URL = 'https://baremetalsolution.googleapis.com/'
  MTLS_BASE_URL = 'https://baremetalsolution.mtls.googleapis.com/'

  _PACKAGE = 'baremetalsolution'
  _SCOPES = ['https://www.googleapis.com/auth/cloud-platform']
  _VERSION = 'v2'
  _CLIENT_ID = 'CLIENT_ID'
  _CLIENT_SECRET = 'CLIENT_SECRET'
  _USER_AGENT = 'google-cloud-sdk'
  _CLIENT_CLASS_NAME = 'BaremetalsolutionV2'
  _URL_VERSION = 'v2'
  _API_KEY = None

  def __init__(self, url='', credentials=None,
               get_credentials=True, http=None, model=None,
               log_request=False, log_response=False,
               credentials_args=None, default_global_params=None,
               additional_http_headers=None, response_encoding=None):
    """Create a new baremetalsolution handle."""
    url = url or self.BASE_URL
    super(BaremetalsolutionV2, self).__init__(
        url, credentials=credentials,
        get_credentials=get_credentials, http=http, model=model,
        log_request=log_request, log_response=log_response,
        credentials_args=credentials_args,
        default_global_params=default_global_params,
        additional_http_headers=additional_http_headers,
        response_encoding=response_encoding)
    self.projects_locations_instanceQuotas = self.ProjectsLocationsInstanceQuotasService(self)
    self.projects_locations_instances = self.ProjectsLocationsInstancesService(self)
    self.projects_locations_networkQuotas = self.ProjectsLocationsNetworkQuotasService(self)
    self.projects_locations_networks = self.ProjectsLocationsNetworksService(self)
    self.projects_locations_nfsShares = self.ProjectsLocationsNfsSharesService(self)
    self.projects_locations_operations = self.ProjectsLocationsOperationsService(self)
    self.projects_locations_osImages = self.ProjectsLocationsOsImagesService(self)
    self.projects_locations_provisioningConfigs = self.ProjectsLocationsProvisioningConfigsService(self)
    self.projects_locations_provisioningQuotas = self.ProjectsLocationsProvisioningQuotasService(self)
    self.projects_locations_snapshotSchedulePolicies = self.ProjectsLocationsSnapshotSchedulePoliciesService(self)
    self.projects_locations_sshKeys = self.ProjectsLocationsSshKeysService(self)
    self.projects_locations_storageQuotas = self.ProjectsLocationsStorageQuotasService(self)
    self.projects_locations_volumes_luns = self.ProjectsLocationsVolumesLunsService(self)
    self.projects_locations_volumes_snapshots = self.ProjectsLocationsVolumesSnapshotsService(self)
    self.projects_locations_volumes = self.ProjectsLocationsVolumesService(self)
    self.projects_locations = self.ProjectsLocationsService(self)
    self.projects = self.ProjectsService(self)

  class ProjectsLocationsInstanceQuotasService(base_api.BaseApiService):
    """Service class for the projects_locations_instanceQuotas resource."""

    _NAME = 'projects_locations_instanceQuotas'

    def __init__(self, client):
      super(BaremetalsolutionV2.ProjectsLocationsInstanceQuotasService, self).__init__(client)
      self._upload_configs = {
          }

    def List(self, request, global_params=None):
      r"""List Instance provisioning quotas.

      Args:
        request: (BaremetalsolutionProjectsLocationsInstanceQuotasListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListInstanceQuotasResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/instanceQuotas',
        http_method='GET',
        method_id='baremetalsolution.projects.locations.instanceQuotas.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v2/{+parent}/instanceQuotas',
        request_field='',
        request_type_name='BaremetalsolutionProjectsLocationsInstanceQuotasListRequest',
        response_type_name='ListInstanceQuotasResponse',
        supports_download=False,
    )

  class ProjectsLocationsInstancesService(base_api.BaseApiService):
    """Service class for the projects_locations_instances resource."""

    _NAME = 'projects_locations_instances'

    def __init__(self, client):
      super(BaremetalsolutionV2.ProjectsLocationsInstancesService, self).__init__(client)
      self._upload_configs = {
          }

    def AttachVolume(self, request, global_params=None):
      r"""Attach volume to instance.

      Args:
        request: (BaremetalsolutionProjectsLocationsInstancesAttachVolumeRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('AttachVolume')
      return self._RunMethod(
          config, request, global_params=global_params)

    AttachVolume.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}:attachVolume',
        http_method='POST',
        method_id='baremetalsolution.projects.locations.instances.attachVolume',
        ordered_params=['instance'],
        path_params=['instance'],
        query_params=[],
        relative_path='v2/{+instance}:attachVolume',
        request_field='attachVolumeRequest',
        request_type_name='BaremetalsolutionProjectsLocationsInstancesAttachVolumeRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def DetachLun(self, request, global_params=None):
      r"""Detach LUN from Instance.

      Args:
        request: (BaremetalsolutionProjectsLocationsInstancesDetachLunRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('DetachLun')
      return self._RunMethod(
          config, request, global_params=global_params)

    DetachLun.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}:detachLun',
        http_method='POST',
        method_id='baremetalsolution.projects.locations.instances.detachLun',
        ordered_params=['instance'],
        path_params=['instance'],
        query_params=[],
        relative_path='v2/{+instance}:detachLun',
        request_field='detachLunRequest',
        request_type_name='BaremetalsolutionProjectsLocationsInstancesDetachLunRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def DetachVolume(self, request, global_params=None):
      r"""Detach volume from instance.

      Args:
        request: (BaremetalsolutionProjectsLocationsInstancesDetachVolumeRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('DetachVolume')
      return self._RunMethod(
          config, request, global_params=global_params)

    DetachVolume.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}:detachVolume',
        http_method='POST',
        method_id='baremetalsolution.projects.locations.instances.detachVolume',
        ordered_params=['instance'],
        path_params=['instance'],
        query_params=[],
        relative_path='v2/{+instance}:detachVolume',
        request_field='detachVolumeRequest',
        request_type_name='BaremetalsolutionProjectsLocationsInstancesDetachVolumeRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def DisableHyperthreading(self, request, global_params=None):
      r"""Perform disable hyperthreading operation on a single server.

      Args:
        request: (BaremetalsolutionProjectsLocationsInstancesDisableHyperthreadingRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('DisableHyperthreading')
      return self._RunMethod(
          config, request, global_params=global_params)

    DisableHyperthreading.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}:disableHyperthreading',
        http_method='POST',
        method_id='baremetalsolution.projects.locations.instances.disableHyperthreading',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}:disableHyperthreading',
        request_field='disableHyperthreadingRequest',
        request_type_name='BaremetalsolutionProjectsLocationsInstancesDisableHyperthreadingRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def DisableInteractiveSerialConsole(self, request, global_params=None):
      r"""Disable the interactive serial console feature on an instance.

      Args:
        request: (BaremetalsolutionProjectsLocationsInstancesDisableInteractiveSerialConsoleRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('DisableInteractiveSerialConsole')
      return self._RunMethod(
          config, request, global_params=global_params)

    DisableInteractiveSerialConsole.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}:disableInteractiveSerialConsole',
        http_method='POST',
        method_id='baremetalsolution.projects.locations.instances.disableInteractiveSerialConsole',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}:disableInteractiveSerialConsole',
        request_field='disableInteractiveSerialConsoleRequest',
        request_type_name='BaremetalsolutionProjectsLocationsInstancesDisableInteractiveSerialConsoleRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def EnableHyperthreading(self, request, global_params=None):
      r"""Perform enable hyperthreading operation on a single server.

      Args:
        request: (BaremetalsolutionProjectsLocationsInstancesEnableHyperthreadingRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('EnableHyperthreading')
      return self._RunMethod(
          config, request, global_params=global_params)

    EnableHyperthreading.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}:enableHyperthreading',
        http_method='POST',
        method_id='baremetalsolution.projects.locations.instances.enableHyperthreading',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}:enableHyperthreading',
        request_field='enableHyperthreadingRequest',
        request_type_name='BaremetalsolutionProjectsLocationsInstancesEnableHyperthreadingRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def EnableInteractiveSerialConsole(self, request, global_params=None):
      r"""Enable the interactive serial console feature on an instance.

      Args:
        request: (BaremetalsolutionProjectsLocationsInstancesEnableInteractiveSerialConsoleRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('EnableInteractiveSerialConsole')
      return self._RunMethod(
          config, request, global_params=global_params)

    EnableInteractiveSerialConsole.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}:enableInteractiveSerialConsole',
        http_method='POST',
        method_id='baremetalsolution.projects.locations.instances.enableInteractiveSerialConsole',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}:enableInteractiveSerialConsole',
        request_field='enableInteractiveSerialConsoleRequest',
        request_type_name='BaremetalsolutionProjectsLocationsInstancesEnableInteractiveSerialConsoleRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Get details about a single server.

      Args:
        request: (BaremetalsolutionProjectsLocationsInstancesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Instance) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}',
        http_method='GET',
        method_id='baremetalsolution.projects.locations.instances.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}',
        request_field='',
        request_type_name='BaremetalsolutionProjectsLocationsInstancesGetRequest',
        response_type_name='Instance',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""List servers in a given project and location.

      Args:
        request: (BaremetalsolutionProjectsLocationsInstancesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListInstancesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/instances',
        http_method='GET',
        method_id='baremetalsolution.projects.locations.instances.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v2/{+parent}/instances',
        request_field='',
        request_type_name='BaremetalsolutionProjectsLocationsInstancesListRequest',
        response_type_name='ListInstancesResponse',
        supports_download=False,
    )

    def LoadAuthInfo(self, request, global_params=None):
      r"""Load auth info for a server.

      Args:
        request: (BaremetalsolutionProjectsLocationsInstancesLoadAuthInfoRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (LoadInstanceAuthInfoResponse) The response message.
      """
      config = self.GetMethodConfig('LoadAuthInfo')
      return self._RunMethod(
          config, request, global_params=global_params)

    LoadAuthInfo.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}:loadAuthInfo',
        http_method='GET',
        method_id='baremetalsolution.projects.locations.instances.loadAuthInfo',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}:loadAuthInfo',
        request_field='',
        request_type_name='BaremetalsolutionProjectsLocationsInstancesLoadAuthInfoRequest',
        response_type_name='LoadInstanceAuthInfoResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Update details of a single server.

      Args:
        request: (BaremetalsolutionProjectsLocationsInstancesPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}',
        http_method='PATCH',
        method_id='baremetalsolution.projects.locations.instances.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v2/{+name}',
        request_field='instance',
        request_type_name='BaremetalsolutionProjectsLocationsInstancesPatchRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Reimage(self, request, global_params=None):
      r"""Perform reimage operation on a single server.

      Args:
        request: (BaremetalsolutionProjectsLocationsInstancesReimageRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Reimage')
      return self._RunMethod(
          config, request, global_params=global_params)

    Reimage.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}:reimage',
        http_method='POST',
        method_id='baremetalsolution.projects.locations.instances.reimage',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}:reimage',
        request_field='reimageInstanceRequest',
        request_type_name='BaremetalsolutionProjectsLocationsInstancesReimageRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Rename(self, request, global_params=None):
      r"""RenameInstance sets a new name for an instance. Use with caution, previous names become immediately invalidated.

      Args:
        request: (BaremetalsolutionProjectsLocationsInstancesRenameRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Instance) The response message.
      """
      config = self.GetMethodConfig('Rename')
      return self._RunMethod(
          config, request, global_params=global_params)

    Rename.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}:rename',
        http_method='POST',
        method_id='baremetalsolution.projects.locations.instances.rename',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}:rename',
        request_field='renameInstanceRequest',
        request_type_name='BaremetalsolutionProjectsLocationsInstancesRenameRequest',
        response_type_name='Instance',
        supports_download=False,
    )

    def Reset(self, request, global_params=None):
      r"""Perform an ungraceful, hard reset on a server. Equivalent to shutting the power off and then turning it back on.

      Args:
        request: (BaremetalsolutionProjectsLocationsInstancesResetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Reset')
      return self._RunMethod(
          config, request, global_params=global_params)

    Reset.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}:reset',
        http_method='POST',
        method_id='baremetalsolution.projects.locations.instances.reset',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}:reset',
        request_field='resetInstanceRequest',
        request_type_name='BaremetalsolutionProjectsLocationsInstancesResetRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Start(self, request, global_params=None):
      r"""Starts a server that was shutdown.

      Args:
        request: (BaremetalsolutionProjectsLocationsInstancesStartRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Start')
      return self._RunMethod(
          config, request, global_params=global_params)

    Start.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}:start',
        http_method='POST',
        method_id='baremetalsolution.projects.locations.instances.start',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}:start',
        request_field='startInstanceRequest',
        request_type_name='BaremetalsolutionProjectsLocationsInstancesStartRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Stop(self, request, global_params=None):
      r"""Stop a running server.

      Args:
        request: (BaremetalsolutionProjectsLocationsInstancesStopRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Stop')
      return self._RunMethod(
          config, request, global_params=global_params)

    Stop.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}:stop',
        http_method='POST',
        method_id='baremetalsolution.projects.locations.instances.stop',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}:stop',
        request_field='stopInstanceRequest',
        request_type_name='BaremetalsolutionProjectsLocationsInstancesStopRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class ProjectsLocationsNetworkQuotasService(base_api.BaseApiService):
    """Service class for the projects_locations_networkQuotas resource."""

    _NAME = 'projects_locations_networkQuotas'

    def __init__(self, client):
      super(BaremetalsolutionV2.ProjectsLocationsNetworkQuotasService, self).__init__(client)
      self._upload_configs = {
          }

    def List(self, request, global_params=None):
      r"""List Network provisioning quotas.

      Args:
        request: (BaremetalsolutionProjectsLocationsNetworkQuotasListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListNetworkQuotasResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/networkQuotas',
        http_method='GET',
        method_id='baremetalsolution.projects.locations.networkQuotas.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v2/{+parent}/networkQuotas',
        request_field='',
        request_type_name='BaremetalsolutionProjectsLocationsNetworkQuotasListRequest',
        response_type_name='ListNetworkQuotasResponse',
        supports_download=False,
    )

  class ProjectsLocationsNetworksService(base_api.BaseApiService):
    """Service class for the projects_locations_networks resource."""

    _NAME = 'projects_locations_networks'

    def __init__(self, client):
      super(BaremetalsolutionV2.ProjectsLocationsNetworksService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Get details of a single network.

      Args:
        request: (BaremetalsolutionProjectsLocationsNetworksGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Network) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/networks/{networksId}',
        http_method='GET',
        method_id='baremetalsolution.projects.locations.networks.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}',
        request_field='',
        request_type_name='BaremetalsolutionProjectsLocationsNetworksGetRequest',
        response_type_name='Network',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""List network in a given project and location.

      Args:
        request: (BaremetalsolutionProjectsLocationsNetworksListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListNetworksResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/networks',
        http_method='GET',
        method_id='baremetalsolution.projects.locations.networks.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v2/{+parent}/networks',
        request_field='',
        request_type_name='BaremetalsolutionProjectsLocationsNetworksListRequest',
        response_type_name='ListNetworksResponse',
        supports_download=False,
    )

    def ListNetworkUsage(self, request, global_params=None):
      r"""List all Networks (and used IPs for each Network) in the vendor account associated with the specified project.

      Args:
        request: (BaremetalsolutionProjectsLocationsNetworksListNetworkUsageRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListNetworkUsageResponse) The response message.
      """
      config = self.GetMethodConfig('ListNetworkUsage')
      return self._RunMethod(
          config, request, global_params=global_params)

    ListNetworkUsage.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/networks:listNetworkUsage',
        http_method='GET',
        method_id='baremetalsolution.projects.locations.networks.listNetworkUsage',
        ordered_params=['location'],
        path_params=['location'],
        query_params=[],
        relative_path='v2/{+location}/networks:listNetworkUsage',
        request_field='',
        request_type_name='BaremetalsolutionProjectsLocationsNetworksListNetworkUsageRequest',
        response_type_name='ListNetworkUsageResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Update details of a single network.

      Args:
        request: (BaremetalsolutionProjectsLocationsNetworksPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/networks/{networksId}',
        http_method='PATCH',
        method_id='baremetalsolution.projects.locations.networks.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v2/{+name}',
        request_field='network',
        request_type_name='BaremetalsolutionProjectsLocationsNetworksPatchRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Rename(self, request, global_params=None):
      r"""RenameNetwork sets a new name for a network. Use with caution, previous names become immediately invalidated.

      Args:
        request: (BaremetalsolutionProjectsLocationsNetworksRenameRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Network) The response message.
      """
      config = self.GetMethodConfig('Rename')
      return self._RunMethod(
          config, request, global_params=global_params)

    Rename.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/networks/{networksId}:rename',
        http_method='POST',
        method_id='baremetalsolution.projects.locations.networks.rename',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}:rename',
        request_field='renameNetworkRequest',
        request_type_name='BaremetalsolutionProjectsLocationsNetworksRenameRequest',
        response_type_name='Network',
        supports_download=False,
    )

  class ProjectsLocationsNfsSharesService(base_api.BaseApiService):
    """Service class for the projects_locations_nfsShares resource."""

    _NAME = 'projects_locations_nfsShares'

    def __init__(self, client):
      super(BaremetalsolutionV2.ProjectsLocationsNfsSharesService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Create an NFS share.

      Args:
        request: (BaremetalsolutionProjectsLocationsNfsSharesCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/nfsShares',
        http_method='POST',
        method_id='baremetalsolution.projects.locations.nfsShares.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=[],
        relative_path='v2/{+parent}/nfsShares',
        request_field='nfsShare',
        request_type_name='BaremetalsolutionProjectsLocationsNfsSharesCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Delete an NFS share. The underlying volume is automatically deleted.

      Args:
        request: (BaremetalsolutionProjectsLocationsNfsSharesDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/nfsShares/{nfsSharesId}',
        http_method='DELETE',
        method_id='baremetalsolution.projects.locations.nfsShares.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}',
        request_field='',
        request_type_name='BaremetalsolutionProjectsLocationsNfsSharesDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Get details of a single NFS share.

      Args:
        request: (BaremetalsolutionProjectsLocationsNfsSharesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (NfsShare) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/nfsShares/{nfsSharesId}',
        http_method='GET',
        method_id='baremetalsolution.projects.locations.nfsShares.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}',
        request_field='',
        request_type_name='BaremetalsolutionProjectsLocationsNfsSharesGetRequest',
        response_type_name='NfsShare',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""List NFS shares.

      Args:
        request: (BaremetalsolutionProjectsLocationsNfsSharesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListNfsSharesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/nfsShares',
        http_method='GET',
        method_id='baremetalsolution.projects.locations.nfsShares.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v2/{+parent}/nfsShares',
        request_field='',
        request_type_name='BaremetalsolutionProjectsLocationsNfsSharesListRequest',
        response_type_name='ListNfsSharesResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Update details of a single NFS share.

      Args:
        request: (BaremetalsolutionProjectsLocationsNfsSharesPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/nfsShares/{nfsSharesId}',
        http_method='PATCH',
        method_id='baremetalsolution.projects.locations.nfsShares.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v2/{+name}',
        request_field='nfsShare',
        request_type_name='BaremetalsolutionProjectsLocationsNfsSharesPatchRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Rename(self, request, global_params=None):
      r"""RenameNfsShare sets a new name for an nfsshare. Use with caution, previous names become immediately invalidated.

      Args:
        request: (BaremetalsolutionProjectsLocationsNfsSharesRenameRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (NfsShare) The response message.
      """
      config = self.GetMethodConfig('Rename')
      return self._RunMethod(
          config, request, global_params=global_params)

    Rename.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/nfsShares/{nfsSharesId}:rename',
        http_method='POST',
        method_id='baremetalsolution.projects.locations.nfsShares.rename',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}:rename',
        request_field='renameNfsShareRequest',
        request_type_name='BaremetalsolutionProjectsLocationsNfsSharesRenameRequest',
        response_type_name='NfsShare',
        supports_download=False,
    )

  class ProjectsLocationsOperationsService(base_api.BaseApiService):
    """Service class for the projects_locations_operations resource."""

    _NAME = 'projects_locations_operations'

    def __init__(self, client):
      super(BaremetalsolutionV2.ProjectsLocationsOperationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Get details about an operation.

      Args:
        request: (BaremetalsolutionProjectsLocationsOperationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='GET',
        method_id='baremetalsolution.projects.locations.operations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}',
        request_field='',
        request_type_name='BaremetalsolutionProjectsLocationsOperationsGetRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class ProjectsLocationsOsImagesService(base_api.BaseApiService):
    """Service class for the projects_locations_osImages resource."""

    _NAME = 'projects_locations_osImages'

    def __init__(self, client):
      super(BaremetalsolutionV2.ProjectsLocationsOsImagesService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Get details of a single OS image.

      Args:
        request: (BaremetalsolutionProjectsLocationsOsImagesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (OSImage) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/osImages/{osImagesId}',
        http_method='GET',
        method_id='baremetalsolution.projects.locations.osImages.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}',
        request_field='',
        request_type_name='BaremetalsolutionProjectsLocationsOsImagesGetRequest',
        response_type_name='OSImage',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Retrieves the list of OS images which are currently approved.

      Args:
        request: (BaremetalsolutionProjectsLocationsOsImagesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListOSImagesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/osImages',
        http_method='GET',
        method_id='baremetalsolution.projects.locations.osImages.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v2/{+parent}/osImages',
        request_field='',
        request_type_name='BaremetalsolutionProjectsLocationsOsImagesListRequest',
        response_type_name='ListOSImagesResponse',
        supports_download=False,
    )

  class ProjectsLocationsProvisioningConfigsService(base_api.BaseApiService):
    """Service class for the projects_locations_provisioningConfigs resource."""

    _NAME = 'projects_locations_provisioningConfigs'

    def __init__(self, client):
      super(BaremetalsolutionV2.ProjectsLocationsProvisioningConfigsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Create new ProvisioningConfig.

      Args:
        request: (BaremetalsolutionProjectsLocationsProvisioningConfigsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ProvisioningConfig) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/provisioningConfigs',
        http_method='POST',
        method_id='baremetalsolution.projects.locations.provisioningConfigs.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['email'],
        relative_path='v2/{+parent}/provisioningConfigs',
        request_field='provisioningConfig',
        request_type_name='BaremetalsolutionProjectsLocationsProvisioningConfigsCreateRequest',
        response_type_name='ProvisioningConfig',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Get ProvisioningConfig by name.

      Args:
        request: (BaremetalsolutionProjectsLocationsProvisioningConfigsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ProvisioningConfig) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/provisioningConfigs/{provisioningConfigsId}',
        http_method='GET',
        method_id='baremetalsolution.projects.locations.provisioningConfigs.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}',
        request_field='',
        request_type_name='BaremetalsolutionProjectsLocationsProvisioningConfigsGetRequest',
        response_type_name='ProvisioningConfig',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Update existing ProvisioningConfig.

      Args:
        request: (BaremetalsolutionProjectsLocationsProvisioningConfigsPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ProvisioningConfig) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/provisioningConfigs/{provisioningConfigsId}',
        http_method='PATCH',
        method_id='baremetalsolution.projects.locations.provisioningConfigs.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['email', 'updateMask'],
        relative_path='v2/{+name}',
        request_field='provisioningConfig',
        request_type_name='BaremetalsolutionProjectsLocationsProvisioningConfigsPatchRequest',
        response_type_name='ProvisioningConfig',
        supports_download=False,
    )

    def Submit(self, request, global_params=None):
      r"""Submit a provisioning configuration for a given project.

      Args:
        request: (BaremetalsolutionProjectsLocationsProvisioningConfigsSubmitRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SubmitProvisioningConfigResponse) The response message.
      """
      config = self.GetMethodConfig('Submit')
      return self._RunMethod(
          config, request, global_params=global_params)

    Submit.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/provisioningConfigs:submit',
        http_method='POST',
        method_id='baremetalsolution.projects.locations.provisioningConfigs.submit',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=[],
        relative_path='v2/{+parent}/provisioningConfigs:submit',
        request_field='submitProvisioningConfigRequest',
        request_type_name='BaremetalsolutionProjectsLocationsProvisioningConfigsSubmitRequest',
        response_type_name='SubmitProvisioningConfigResponse',
        supports_download=False,
    )

  class ProjectsLocationsProvisioningQuotasService(base_api.BaseApiService):
    """Service class for the projects_locations_provisioningQuotas resource."""

    _NAME = 'projects_locations_provisioningQuotas'

    def __init__(self, client):
      super(BaremetalsolutionV2.ProjectsLocationsProvisioningQuotasService, self).__init__(client)
      self._upload_configs = {
          }

    def List(self, request, global_params=None):
      r"""List the budget details to provision resources on a given project.

      Args:
        request: (BaremetalsolutionProjectsLocationsProvisioningQuotasListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListProvisioningQuotasResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/provisioningQuotas',
        http_method='GET',
        method_id='baremetalsolution.projects.locations.provisioningQuotas.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v2/{+parent}/provisioningQuotas',
        request_field='',
        request_type_name='BaremetalsolutionProjectsLocationsProvisioningQuotasListRequest',
        response_type_name='ListProvisioningQuotasResponse',
        supports_download=False,
    )

  class ProjectsLocationsSnapshotSchedulePoliciesService(base_api.BaseApiService):
    """Service class for the projects_locations_snapshotSchedulePolicies resource."""

    _NAME = 'projects_locations_snapshotSchedulePolicies'

    def __init__(self, client):
      super(BaremetalsolutionV2.ProjectsLocationsSnapshotSchedulePoliciesService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Create a snapshot schedule policy in the specified project.

      Args:
        request: (BaremetalsolutionProjectsLocationsSnapshotSchedulePoliciesCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SnapshotSchedulePolicy) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/snapshotSchedulePolicies',
        http_method='POST',
        method_id='baremetalsolution.projects.locations.snapshotSchedulePolicies.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['snapshotSchedulePolicyId'],
        relative_path='v2/{+parent}/snapshotSchedulePolicies',
        request_field='snapshotSchedulePolicy',
        request_type_name='BaremetalsolutionProjectsLocationsSnapshotSchedulePoliciesCreateRequest',
        response_type_name='SnapshotSchedulePolicy',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Delete a named snapshot schedule policy.

      Args:
        request: (BaremetalsolutionProjectsLocationsSnapshotSchedulePoliciesDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/snapshotSchedulePolicies/{snapshotSchedulePoliciesId}',
        http_method='DELETE',
        method_id='baremetalsolution.projects.locations.snapshotSchedulePolicies.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}',
        request_field='',
        request_type_name='BaremetalsolutionProjectsLocationsSnapshotSchedulePoliciesDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Get details of a single snapshot schedule policy.

      Args:
        request: (BaremetalsolutionProjectsLocationsSnapshotSchedulePoliciesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SnapshotSchedulePolicy) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/snapshotSchedulePolicies/{snapshotSchedulePoliciesId}',
        http_method='GET',
        method_id='baremetalsolution.projects.locations.snapshotSchedulePolicies.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}',
        request_field='',
        request_type_name='BaremetalsolutionProjectsLocationsSnapshotSchedulePoliciesGetRequest',
        response_type_name='SnapshotSchedulePolicy',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""List snapshot schedule policies in a given project and location.

      Args:
        request: (BaremetalsolutionProjectsLocationsSnapshotSchedulePoliciesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListSnapshotSchedulePoliciesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/snapshotSchedulePolicies',
        http_method='GET',
        method_id='baremetalsolution.projects.locations.snapshotSchedulePolicies.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v2/{+parent}/snapshotSchedulePolicies',
        request_field='',
        request_type_name='BaremetalsolutionProjectsLocationsSnapshotSchedulePoliciesListRequest',
        response_type_name='ListSnapshotSchedulePoliciesResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Update a snapshot schedule policy in the specified project.

      Args:
        request: (BaremetalsolutionProjectsLocationsSnapshotSchedulePoliciesPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SnapshotSchedulePolicy) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/snapshotSchedulePolicies/{snapshotSchedulePoliciesId}',
        http_method='PATCH',
        method_id='baremetalsolution.projects.locations.snapshotSchedulePolicies.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v2/{+name}',
        request_field='snapshotSchedulePolicy',
        request_type_name='BaremetalsolutionProjectsLocationsSnapshotSchedulePoliciesPatchRequest',
        response_type_name='SnapshotSchedulePolicy',
        supports_download=False,
    )

  class ProjectsLocationsSshKeysService(base_api.BaseApiService):
    """Service class for the projects_locations_sshKeys resource."""

    _NAME = 'projects_locations_sshKeys'

    def __init__(self, client):
      super(BaremetalsolutionV2.ProjectsLocationsSshKeysService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Register a public SSH key in the specified project for use with the interactive serial console feature.

      Args:
        request: (BaremetalsolutionProjectsLocationsSshKeysCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SSHKey) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/sshKeys',
        http_method='POST',
        method_id='baremetalsolution.projects.locations.sshKeys.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['sshKeyId'],
        relative_path='v2/{+parent}/sshKeys',
        request_field='sSHKey',
        request_type_name='BaremetalsolutionProjectsLocationsSshKeysCreateRequest',
        response_type_name='SSHKey',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a public SSH key registered in the specified project.

      Args:
        request: (BaremetalsolutionProjectsLocationsSshKeysDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/sshKeys/{sshKeysId}',
        http_method='DELETE',
        method_id='baremetalsolution.projects.locations.sshKeys.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}',
        request_field='',
        request_type_name='BaremetalsolutionProjectsLocationsSshKeysDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists the public SSH keys registered for the specified project. These SSH keys are used only for the interactive serial console feature.

      Args:
        request: (BaremetalsolutionProjectsLocationsSshKeysListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListSSHKeysResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/sshKeys',
        http_method='GET',
        method_id='baremetalsolution.projects.locations.sshKeys.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v2/{+parent}/sshKeys',
        request_field='',
        request_type_name='BaremetalsolutionProjectsLocationsSshKeysListRequest',
        response_type_name='ListSSHKeysResponse',
        supports_download=False,
    )

  class ProjectsLocationsStorageQuotasService(base_api.BaseApiService):
    """Service class for the projects_locations_storageQuotas resource."""

    _NAME = 'projects_locations_storageQuotas'

    def __init__(self, client):
      super(BaremetalsolutionV2.ProjectsLocationsStorageQuotasService, self).__init__(client)
      self._upload_configs = {
          }

    def List(self, request, global_params=None):
      r"""List Storage provisioning quotas.

      Args:
        request: (BaremetalsolutionProjectsLocationsStorageQuotasListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListStorageQuotasResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/storageQuotas',
        http_method='GET',
        method_id='baremetalsolution.projects.locations.storageQuotas.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v2/{+parent}/storageQuotas',
        request_field='',
        request_type_name='BaremetalsolutionProjectsLocationsStorageQuotasListRequest',
        response_type_name='ListStorageQuotasResponse',
        supports_download=False,
    )

  class ProjectsLocationsVolumesLunsService(base_api.BaseApiService):
    """Service class for the projects_locations_volumes_luns resource."""

    _NAME = 'projects_locations_volumes_luns'

    def __init__(self, client):
      super(BaremetalsolutionV2.ProjectsLocationsVolumesLunsService, self).__init__(client)
      self._upload_configs = {
          }

    def Delete(self, request, global_params=None):
      r"""Delete a Lun. Lun shouldn't be attached to any Instances.

      Args:
        request: (BaremetalsolutionProjectsLocationsVolumesLunsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/volumes/{volumesId}/luns/{lunsId}',
        http_method='DELETE',
        method_id='baremetalsolution.projects.locations.volumes.luns.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}',
        request_field='',
        request_type_name='BaremetalsolutionProjectsLocationsVolumesLunsDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Evict(self, request, global_params=None):
      r"""Skips lun's cooloff and deletes it now. Lun must be in cooloff state.

      Args:
        request: (BaremetalsolutionProjectsLocationsVolumesLunsEvictRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Evict')
      return self._RunMethod(
          config, request, global_params=global_params)

    Evict.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/volumes/{volumesId}/luns/{lunsId}:evict',
        http_method='POST',
        method_id='baremetalsolution.projects.locations.volumes.luns.evict',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}:evict',
        request_field='evictLunRequest',
        request_type_name='BaremetalsolutionProjectsLocationsVolumesLunsEvictRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Get details of a single storage logical unit number(LUN).

      Args:
        request: (BaremetalsolutionProjectsLocationsVolumesLunsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Lun) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/volumes/{volumesId}/luns/{lunsId}',
        http_method='GET',
        method_id='baremetalsolution.projects.locations.volumes.luns.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}',
        request_field='',
        request_type_name='BaremetalsolutionProjectsLocationsVolumesLunsGetRequest',
        response_type_name='Lun',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""List storage volume luns for given storage volume.

      Args:
        request: (BaremetalsolutionProjectsLocationsVolumesLunsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListLunsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/volumes/{volumesId}/luns',
        http_method='GET',
        method_id='baremetalsolution.projects.locations.volumes.luns.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v2/{+parent}/luns',
        request_field='',
        request_type_name='BaremetalsolutionProjectsLocationsVolumesLunsListRequest',
        response_type_name='ListLunsResponse',
        supports_download=False,
    )

    def OnDemandEvict(self, request, global_params=None):
      r"""Skips lun's cooloff and deletes it now. Lun must be in cooloff state. This method is used for Evict done with PAM.

      Args:
        request: (BaremetalsolutionProjectsLocationsVolumesLunsOnDemandEvictRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('OnDemandEvict')
      return self._RunMethod(
          config, request, global_params=global_params)

    OnDemandEvict.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/volumes/{volumesId}/luns/{lunsId}:onDemandEvict',
        http_method='POST',
        method_id='baremetalsolution.projects.locations.volumes.luns.onDemandEvict',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}:onDemandEvict',
        request_field='onDemandEvictLunRequest',
        request_type_name='BaremetalsolutionProjectsLocationsVolumesLunsOnDemandEvictRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class ProjectsLocationsVolumesSnapshotsService(base_api.BaseApiService):
    """Service class for the projects_locations_volumes_snapshots resource."""

    _NAME = 'projects_locations_volumes_snapshots'

    def __init__(self, client):
      super(BaremetalsolutionV2.ProjectsLocationsVolumesSnapshotsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Takes a snapshot of a boot volume. Returns INVALID_ARGUMENT if called for a non-boot volume.

      Args:
        request: (BaremetalsolutionProjectsLocationsVolumesSnapshotsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (VolumeSnapshot) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/volumes/{volumesId}/snapshots',
        http_method='POST',
        method_id='baremetalsolution.projects.locations.volumes.snapshots.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=[],
        relative_path='v2/{+parent}/snapshots',
        request_field='volumeSnapshot',
        request_type_name='BaremetalsolutionProjectsLocationsVolumesSnapshotsCreateRequest',
        response_type_name='VolumeSnapshot',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a volume snapshot. Returns INVALID_ARGUMENT if called for a non-boot volume.

      Args:
        request: (BaremetalsolutionProjectsLocationsVolumesSnapshotsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/volumes/{volumesId}/snapshots/{snapshotsId}',
        http_method='DELETE',
        method_id='baremetalsolution.projects.locations.volumes.snapshots.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}',
        request_field='',
        request_type_name='BaremetalsolutionProjectsLocationsVolumesSnapshotsDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Returns the specified snapshot resource. Returns INVALID_ARGUMENT if called for a non-boot volume.

      Args:
        request: (BaremetalsolutionProjectsLocationsVolumesSnapshotsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (VolumeSnapshot) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/volumes/{volumesId}/snapshots/{snapshotsId}',
        http_method='GET',
        method_id='baremetalsolution.projects.locations.volumes.snapshots.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}',
        request_field='',
        request_type_name='BaremetalsolutionProjectsLocationsVolumesSnapshotsGetRequest',
        response_type_name='VolumeSnapshot',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Retrieves the list of snapshots for the specified volume. Returns a response with an empty list of snapshots if called for a non-boot volume.

      Args:
        request: (BaremetalsolutionProjectsLocationsVolumesSnapshotsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListVolumeSnapshotsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/volumes/{volumesId}/snapshots',
        http_method='GET',
        method_id='baremetalsolution.projects.locations.volumes.snapshots.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v2/{+parent}/snapshots',
        request_field='',
        request_type_name='BaremetalsolutionProjectsLocationsVolumesSnapshotsListRequest',
        response_type_name='ListVolumeSnapshotsResponse',
        supports_download=False,
    )

    def RestoreVolumeSnapshot(self, request, global_params=None):
      r"""Uses the specified snapshot to restore its parent volume. Returns INVALID_ARGUMENT if called for a non-boot volume.

      Args:
        request: (BaremetalsolutionProjectsLocationsVolumesSnapshotsRestoreVolumeSnapshotRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('RestoreVolumeSnapshot')
      return self._RunMethod(
          config, request, global_params=global_params)

    RestoreVolumeSnapshot.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/volumes/{volumesId}/snapshots/{snapshotsId}:restoreVolumeSnapshot',
        http_method='POST',
        method_id='baremetalsolution.projects.locations.volumes.snapshots.restoreVolumeSnapshot',
        ordered_params=['volumeSnapshot'],
        path_params=['volumeSnapshot'],
        query_params=[],
        relative_path='v2/{+volumeSnapshot}:restoreVolumeSnapshot',
        request_field='restoreVolumeSnapshotRequest',
        request_type_name='BaremetalsolutionProjectsLocationsVolumesSnapshotsRestoreVolumeSnapshotRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class ProjectsLocationsVolumesService(base_api.BaseApiService):
    """Service class for the projects_locations_volumes resource."""

    _NAME = 'projects_locations_volumes'

    def __init__(self, client):
      super(BaremetalsolutionV2.ProjectsLocationsVolumesService, self).__init__(client)
      self._upload_configs = {
          }

    def AllocateLuns(self, request, global_params=None):
      r"""Allocate Volume's Luns.

      Args:
        request: (BaremetalsolutionProjectsLocationsVolumesAllocateLunsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('AllocateLuns')
      return self._RunMethod(
          config, request, global_params=global_params)

    AllocateLuns.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/volumes/{volumesId}:allocateLuns',
        http_method='POST',
        method_id='baremetalsolution.projects.locations.volumes.allocateLuns',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=[],
        relative_path='v2/{+parent}:allocateLuns',
        request_field='allocateLunsRequest',
        request_type_name='BaremetalsolutionProjectsLocationsVolumesAllocateLunsRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Create(self, request, global_params=None):
      r"""Create a volume.

      Args:
        request: (BaremetalsolutionProjectsLocationsVolumesCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/volumes',
        http_method='POST',
        method_id='baremetalsolution.projects.locations.volumes.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=[],
        relative_path='v2/{+parent}/volumes',
        request_field='volume',
        request_type_name='BaremetalsolutionProjectsLocationsVolumesCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def CreateAndAttach(self, request, global_params=None):
      r"""Create a volume, allocate Luns and attach them to instances.

      Args:
        request: (BaremetalsolutionProjectsLocationsVolumesCreateAndAttachRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('CreateAndAttach')
      return self._RunMethod(
          config, request, global_params=global_params)

    CreateAndAttach.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/volumes:createAndAttach',
        http_method='POST',
        method_id='baremetalsolution.projects.locations.volumes.createAndAttach',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=[],
        relative_path='v2/{+parent}/volumes:createAndAttach',
        request_field='createAndAttachVolumeRequest',
        request_type_name='BaremetalsolutionProjectsLocationsVolumesCreateAndAttachRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Delete a volume. Volume shouldn't have any Luns.

      Args:
        request: (BaremetalsolutionProjectsLocationsVolumesDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/volumes/{volumesId}',
        http_method='DELETE',
        method_id='baremetalsolution.projects.locations.volumes.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['force'],
        relative_path='v2/{+name}',
        request_field='',
        request_type_name='BaremetalsolutionProjectsLocationsVolumesDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Evict(self, request, global_params=None):
      r"""Skips volume's cooloff and deletes it now. Volume must be in cooloff state.

      Args:
        request: (BaremetalsolutionProjectsLocationsVolumesEvictRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Evict')
      return self._RunMethod(
          config, request, global_params=global_params)

    Evict.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/volumes/{volumesId}:evict',
        http_method='POST',
        method_id='baremetalsolution.projects.locations.volumes.evict',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}:evict',
        request_field='evictVolumeRequest',
        request_type_name='BaremetalsolutionProjectsLocationsVolumesEvictRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Get details of a single storage volume.

      Args:
        request: (BaremetalsolutionProjectsLocationsVolumesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Volume) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/volumes/{volumesId}',
        http_method='GET',
        method_id='baremetalsolution.projects.locations.volumes.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}',
        request_field='',
        request_type_name='BaremetalsolutionProjectsLocationsVolumesGetRequest',
        response_type_name='Volume',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""List storage volumes in a given project and location.

      Args:
        request: (BaremetalsolutionProjectsLocationsVolumesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListVolumesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/volumes',
        http_method='GET',
        method_id='baremetalsolution.projects.locations.volumes.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v2/{+parent}/volumes',
        request_field='',
        request_type_name='BaremetalsolutionProjectsLocationsVolumesListRequest',
        response_type_name='ListVolumesResponse',
        supports_download=False,
    )

    def OnDemandEvict(self, request, global_params=None):
      r"""Skips volume's cooloff and deletes it now. Volume must be in cooloff state. This method is used for Evict done with PAM.

      Args:
        request: (BaremetalsolutionProjectsLocationsVolumesOnDemandEvictRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('OnDemandEvict')
      return self._RunMethod(
          config, request, global_params=global_params)

    OnDemandEvict.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/volumes/{volumesId}:onDemandEvict',
        http_method='POST',
        method_id='baremetalsolution.projects.locations.volumes.onDemandEvict',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}:onDemandEvict',
        request_field='onDemandEvictVolumeRequest',
        request_type_name='BaremetalsolutionProjectsLocationsVolumesOnDemandEvictRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Update details of a single storage volume.

      Args:
        request: (BaremetalsolutionProjectsLocationsVolumesPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/volumes/{volumesId}',
        http_method='PATCH',
        method_id='baremetalsolution.projects.locations.volumes.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v2/{+name}',
        request_field='volume',
        request_type_name='BaremetalsolutionProjectsLocationsVolumesPatchRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Rename(self, request, global_params=None):
      r"""RenameVolume sets a new name for a volume. Use with caution, previous names become immediately invalidated.

      Args:
        request: (BaremetalsolutionProjectsLocationsVolumesRenameRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Volume) The response message.
      """
      config = self.GetMethodConfig('Rename')
      return self._RunMethod(
          config, request, global_params=global_params)

    Rename.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/volumes/{volumesId}:rename',
        http_method='POST',
        method_id='baremetalsolution.projects.locations.volumes.rename',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}:rename',
        request_field='renameVolumeRequest',
        request_type_name='BaremetalsolutionProjectsLocationsVolumesRenameRequest',
        response_type_name='Volume',
        supports_download=False,
    )

    def Resize(self, request, global_params=None):
      r"""Emergency Volume resize.

      Args:
        request: (BaremetalsolutionProjectsLocationsVolumesResizeRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Resize')
      return self._RunMethod(
          config, request, global_params=global_params)

    Resize.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}/volumes/{volumesId}:resize',
        http_method='POST',
        method_id='baremetalsolution.projects.locations.volumes.resize',
        ordered_params=['volume'],
        path_params=['volume'],
        query_params=[],
        relative_path='v2/{+volume}:resize',
        request_field='resizeVolumeRequest',
        request_type_name='BaremetalsolutionProjectsLocationsVolumesResizeRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class ProjectsLocationsService(base_api.BaseApiService):
    """Service class for the projects_locations resource."""

    _NAME = 'projects_locations'

    def __init__(self, client):
      super(BaremetalsolutionV2.ProjectsLocationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets information about a location.

      Args:
        request: (BaremetalsolutionProjectsLocationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Location) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations/{locationsId}',
        http_method='GET',
        method_id='baremetalsolution.projects.locations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}',
        request_field='',
        request_type_name='BaremetalsolutionProjectsLocationsGetRequest',
        response_type_name='Location',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists information about the supported locations for this service. This method lists locations based on the resource scope provided in the ListLocationsRequest.name field: * **Global locations**: If `name` is empty, the method lists the public locations available to all projects. * **Project-specific locations**: If `name` follows the format `projects/{project}`, the method lists locations visible to that specific project. This includes public, private, or other project-specific locations enabled for the project. For gRPC and client library implementations, the resource name is passed as the `name` field. For direct service calls, the resource name is incorporated into the request path based on the specific service implementation and version.

      Args:
        request: (BaremetalsolutionProjectsLocationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListLocationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/projects/{projectsId}/locations',
        http_method='GET',
        method_id='baremetalsolution.projects.locations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['extraLocationTypes', 'filter', 'pageSize', 'pageToken'],
        relative_path='v2/{+name}/locations',
        request_field='',
        request_type_name='BaremetalsolutionProjectsLocationsListRequest',
        response_type_name='ListLocationsResponse',
        supports_download=False,
    )

  class ProjectsService(base_api.BaseApiService):
    """Service class for the projects resource."""

    _NAME = 'projects'

    def __init__(self, client):
      super(BaremetalsolutionV2.ProjectsService, self).__init__(client)
      self._upload_configs = {
          }



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Translate module for GAE to Cloud Run conversion.

This module contains the implementation for conversion of App Engine app.yaml or
deployed version to Cloud Run.
"""

from collections.abc import Mapping, Sequence
import os
import re
from typing import Any

from apitools.base.protorpclite import messages
from apitools.base.py import encoding
from googlecloudsdk.api_lib.app import appengine_api_client
from googlecloudsdk.command_lib.app.gae_to_cr_migration_util.common import util
from googlecloudsdk.command_lib.app.gae_to_cr_migration_util.config import feature_helper
from googlecloudsdk.command_lib.app.gae_to_cr_migration_util.translation_rules import concurrent_requests
from googlecloudsdk.command_lib.app.gae_to_cr_migration_util.translation_rules import cpu_memory
from googlecloudsdk.command_lib.app.gae_to_cr_migration_util.translation_rules import entrypoint
from googlecloudsdk.command_lib.app.gae_to_cr_migration_util.translation_rules import health_checks
from googlecloudsdk.command_lib.app.gae_to_cr_migration_util.translation_rules import network
from googlecloudsdk.command_lib.app.gae_to_cr_migration_util.translation_rules import required_flags
from googlecloudsdk.command_lib.app.gae_to_cr_migration_util.translation_rules import scaling
from googlecloudsdk.command_lib.app.gae_to_cr_migration_util.translation_rules import supported_features
from googlecloudsdk.command_lib.app.gae_to_cr_migration_util.translation_rules import timeout
from googlecloudsdk.command_lib.app.gae_to_cr_migration_util.translation_rules import volumes
from googlecloudsdk.core import properties
from googlecloudsdk.core.console import console_io


ExportImageResult = appengine_api_client.ExportImageResult

# Regex patterns used in _to_snake_case
_CAMEL_TO_SNAKE_1 = re.compile(r'(.)([A-Z][a-z]+)')
_CAMEL_TO_SNAKE_2 = re.compile(r'([a-z0-9])([A-Z])')


def _to_snake_case(name: str) -> str:
  """Converts camelCase to snake_case."""
  s1 = _CAMEL_TO_SNAKE_1.sub(r'\1_\2', name)
  return _CAMEL_TO_SNAKE_2.sub(r'\1_\2', s1).lower()


def translate_from_source(
    input_data: Mapping[str, any],
    input_type: feature_helper.InputType,
    appyaml: str,
    service: str,
) -> Sequence[str]:
  """Translates GAE app config to a Cloud Run deploy command.

  This function converts an App Engine app.yaml configuration into a
  `gcloud run deploy` command.

  Args:
    input_data: The original input data, either from app.yaml or the Admin API.
    input_type: The input type of the input data.
    appyaml: The path to the app.yaml file.
    service: The App Engine service to migrate.

  Returns:
    A sequence of strings representing the gcloud run deploy command.
  """
  if not input_data:
    return []
  target_service = (
      service or _get_service_name(input_data)
  )
  if input_type == feature_helper.InputType.ADMIN_API:
    input_flatten_as_appyaml = _convert_admin_api_input_to_app_yaml(input_data)
  else:
    input_flatten_as_appyaml = util.flatten_keys(input_data, parent_path='')

  source_path = get_source_path(feature_helper.InputType.APP_YAML, appyaml)

  flags: Sequence[str] = _get_cloud_run_flags(
      input_data=input_data,
      input_flatten_as_appyaml=input_flatten_as_appyaml,
      source_path=source_path,
      runtime_base_image=None,
  )
  return _generate_output(target_service, flags, source_path, None)


def translate_from_exported_image(
    input_data: Mapping[str, any],
    service: str,
    export_image_response: ExportImageResult,
) -> Sequence[str]:
  """Translates a deployed GAE version to Cloud Run command via image export.

  Args:
    input_data: The original input data, either from app.yaml or the Admin API.
    service: The App Engine service to migrate.
    export_image_response: An ExportImageResult object containing the exported
      image URI, runtime ID, and runtime base image.

  Returns:
    A sequence of strings representing the gcloud run deploy command.
  """
  if not input_data:
    return []
  target_service = (
      service or _get_service_name(input_data)
  )
  input_flatten_as_appyaml = _convert_admin_api_input_to_app_yaml(input_data)
  if export_image_response is not None:
    image = export_image_response.image_uri
    runtime_base_image = export_image_response.runtime_base_image
  else:
    image, runtime_base_image = None, None
  flags: Sequence[str] = _get_cloud_run_flags(
      input_data=input_data,
      input_flatten_as_appyaml=input_flatten_as_appyaml,
      source_path=None,
      runtime_base_image=runtime_base_image,
  )
  return _generate_output(
      target_service,
      flags,
      None,
      image,
  )


def translate_from_image(
    input_data: Mapping[str, any],
    service: str,
) -> Sequence[str]:
  """Translates a GAE app config to a Cloud Run deploy command via AR image.

  Args:
    input_data: The original input data, either from app.yaml or the Admin API.
    service: The App Engine service to migrate.

  Returns:
    A sequence of strings representing the gcloud run deploy command.
  """

  if not input_data:
    return []
  target_service = service or _get_service_name(input_data)
  image = input_data.get(
      'deployment'
  ).container.image  # Get image before flattening
  input_flatten_as_appyaml = _convert_admin_api_input_to_app_yaml(input_data)

  flags = _get_cloud_run_flags(
      input_data=input_data,
      input_flatten_as_appyaml=input_flatten_as_appyaml,
      source_path=None,
      runtime_base_image=None,
  )
  return _generate_output(
      target_service,
      flags,
      None,
      image,
  )


def get_source_path(input_type: feature_helper.InputType, appyaml: str) -> str:
  """Gets the source path for the Cloud Run deploy command."""
  if input_type == feature_helper.InputType.APP_YAML:
    if appyaml:
      source_path = os.path.dirname(appyaml)
      # os.path.dirname('') returns '', so default to '.' in that case.
      return source_path if source_path else '.'
    return '.'
  else:
    return (
        console_io.PromptWithDefault(
            'Is the source code located in the current directory? If not,'
            ' please provide its path relative to the current directory: ',
            default='.',
        )
        + '/'
    )


def _convert_keys_to_snake_case(data: Any) -> Any:
  """Recursively converts keys in dictionaries within the data to snake_case."""
  if isinstance(data, messages.Message):
    data = encoding.MessageToDict(data)
  if isinstance(data, dict):
    new_dict = {}
    for k, v in data.items():
      new_key = _to_snake_case(k)
      new_dict[new_key] = _convert_keys_to_snake_case(v)
    return new_dict
  elif isinstance(data, list):
    return [_convert_keys_to_snake_case(elem) for elem in data]
  else:
    return data


def _convert_admin_api_input_to_app_yaml(
    version_data: Mapping[str, Any]
) -> Mapping[str, Any]:
  """Converts a dict from Admin API Version data to a dict resembling app.yaml.

  Args:
    version_data: A dictionary derived from the App Engine Admin API Version
      object, potentially containing camelCase keys and nested dictionaries.

  Returns:
    A dictionary structured like a parsed app.yaml, with snake_case keys.
  """
  if not version_data:
    return {}

  app_yaml_data = _convert_keys_to_snake_case(version_data)
  flattened_data = util.flatten_keys(app_yaml_data, parent_path='')
  result = {}
  for k, v in flattened_data.items():
    new_key = k.replace('.standard_scheduler_settings', '')
    result[new_key] = v
  return result


def translate_to_service_yaml(
    input_data: Mapping[str, Any],
    input_type: feature_helper.InputType,
    project: str,
    service_name: str | None = None,
) -> dict[str, Any]:
  """Translates App Engine configuration to a Cloud Run service.yaml dictionary.

  Args:
    input_data: The original input data, either from app.yaml or the Admin API.
    input_type: The input type of the input data.
    project: The GCP project ID.
    service_name: The desired Cloud Run service name.

  Returns:
    A dictionary representing the Cloud Run service.yaml.
  """
  if input_type == feature_helper.InputType.ADMIN_API:
    input_flatten_as_appyaml = _convert_admin_api_input_to_app_yaml(input_data)
  else:
    input_flatten_as_appyaml = util.flatten_keys(input_data, parent_path='')

  target_service = service_name or _get_service_name(input_data)
  service_yaml = _get_basic_service_yaml(target_service)

  feature_config = feature_helper.get_feature_config()
  range_limited_features_app_yaml = (
      feature_helper.get_feature_list_by_input_type(
          feature_helper.InputType.APP_YAML, feature_config.range_limited
      )
  )

  if util.is_flex_env(input_data):
    health_checks.update_service_yaml_with_health_checks(
        service_yaml, input_flatten_as_appyaml
    )
  timeout.update_service_yaml_with_timeout(
      service_yaml, input_flatten_as_appyaml
  )
  volumes.update_service_yaml_with_volumes(
      service_yaml, input_flatten_as_appyaml
  )
  concurrent_requests.update_service_yaml_with_concurrent_requests(
      service_yaml, input_flatten_as_appyaml, range_limited_features_app_yaml
  )
  cpu_memory.update_service_yaml_with_cpu_memory(
      service_yaml, input_flatten_as_appyaml
  )
  scaling.update_service_yaml_with_scaling(
      service_yaml, input_flatten_as_appyaml, range_limited_features_app_yaml
  )
  supported_features.update_service_yaml_with_supported_features(
      service_yaml, input_flatten_as_appyaml, project
  )
  network.update_service_yaml_with_network(
      service_yaml, input_flatten_as_appyaml
  )
  required_flags.update_service_yaml_with_required_flags(
      service_yaml, input_data
  )

  entrypoint.update_service_yaml_with_entrypoint(
      service_yaml, input_flatten_as_appyaml
  )

  return service_yaml


def _get_basic_service_yaml(service_name: str) -> dict[str, Any]:
  """Returns a basic service.yaml structure as a dictionary."""
  return {
      'apiVersion': 'serving.knative.dev/v1',
      'kind': 'Service',
      'metadata': {
          'name': service_name,
      },
      'spec': {
          'template': {
              'spec': {
                  'containers': [{}],
              }
          }
      },
  }


def _get_cloud_run_flags(
    input_data: Mapping[str, any],
    input_flatten_as_appyaml: Mapping[str, any],
    source_path: str | None,
    runtime_base_image: str | None,
) -> Sequence[str]:
  """Gets the cloud run flags for the given input data.

  Args:
    input_data: The original input data, either from app.yaml or the Admin API.
    input_flatten_as_appyaml: A flattened mapping of the input data, with keys
      translated to their app.yaml equivalents.
    source_path: The path to the application's source code, if deploying from
      source. None if deploying from an image.
    runtime_base_image: The base image URL for the runtime, if provided during
      image export. None otherwise.

  Returns:
    A sequence of strings representing the Cloud Run flags.
  """

  feature_config = feature_helper.get_feature_config()
  range_limited_features_app_yaml = (
      feature_helper.get_feature_list_by_input_type(
          feature_helper.InputType.APP_YAML, feature_config.range_limited
      )
  )
  supported_features_app_yaml = feature_helper.get_feature_list_by_input_type(
      feature_helper.InputType.APP_YAML, feature_config.supported
  )
  project = properties.VALUES.core.project.Get()

  derived_entrypoint = input_flatten_as_appyaml.get(
      'entrypoint'
  ) or input_flatten_as_appyaml.get('entrypoint.shell')

  if source_path is not None:
    entrypoint_flags = entrypoint.translate_entrypoint_features(
        derived_entrypoint
    )
  elif derived_entrypoint:
    entrypoint_flags = [f'--command={derived_entrypoint}']
  else:
    entrypoint_flags = []

  # We will keep adding flags to this service_yaml as we process the input
  # data, and in the end return the completed service_yaml and remove the
  # corresponding flags from the return value.
  target_service = _get_service_name(input_data)
  service_yaml = _get_basic_service_yaml(target_service)
  health_checks.update_service_yaml_with_health_checks(
      service_yaml, input_flatten_as_appyaml
  )
  timeout.update_service_yaml_with_timeout(
      service_yaml, input_flatten_as_appyaml
  )
  volumes.update_service_yaml_with_volumes(
      service_yaml, input_flatten_as_appyaml
  )
  concurrent_requests.update_service_yaml_with_concurrent_requests(
      service_yaml, input_flatten_as_appyaml, range_limited_features_app_yaml
  )
  cpu_memory.update_service_yaml_with_cpu_memory(
      service_yaml, input_flatten_as_appyaml
  )
  scaling.update_service_yaml_with_scaling(
      service_yaml, input_flatten_as_appyaml, range_limited_features_app_yaml
  )
  supported_features.update_service_yaml_with_supported_features(
      service_yaml,
      input_flatten_as_appyaml,
      project,
  )
  network.update_service_yaml_with_network(
      service_yaml, input_flatten_as_appyaml
  )

  return (
      concurrent_requests.translate_concurrent_requests_features(
          input_flatten_as_appyaml, range_limited_features_app_yaml
      )
      + scaling.translate_scaling_features(
          input_flatten_as_appyaml, range_limited_features_app_yaml
      )
      + timeout.translate_timeout_features(input_flatten_as_appyaml)
      + supported_features.translate_supported_features(
          input_flatten_as_appyaml,
          supported_features_app_yaml,
          project,
      )
      + entrypoint_flags
      + required_flags.translate_add_required_flags(
          input_data, source_path, runtime_base_image
      )
      + volumes.translate_volumes(input_flatten_as_appyaml)
      + cpu_memory.translate_app_resources(input_flatten_as_appyaml)
      + network.translate_network_features(input_flatten_as_appyaml)
      + (
          health_checks.translate_all_health_checks(input_flatten_as_appyaml)
          if util.is_flex_env(input_data)
          else []
      )
  )


def _get_service_name(input_data: Mapping[str, any]) -> str:
  """Gets the service name from the input data."""
  if 'service' in input_data:
    custom_service_name = input_data['service'].strip()
    if custom_service_name:
      return custom_service_name
  return 'default'


def _generate_output(
    service_name: str,
    flags: Sequence[str],
    source_path: str | None,
    image: str | None,
) -> Sequence[str]:
  """Generates the output for the Cloud Run deploy command.

  Args:
    service_name: The name of the Cloud Run service.
    flags: A sequence of flags to include in the gcloud run deploy command.
    source_path: The path to the source code. If provided, the `--source` flag
      will be used.
    image: The URL of the container image. If provided, the `--image` flag will
      be used.

  Returns:
    A sequence of strings representing the gcloud run deploy command.
  """
  output = [
      'gcloud',
      'run',
      'deploy',
      f'{service_name}',
  ]
  if image is not None:
    output.extend([f'--image={image}'])
  elif source_path is not None:
    output.extend([f'--source={source_path}'])
  if flags is not None:
    output.extend(flags)
  return output



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Commands for reading and manipulating snapshot groups."""


from googlecloudsdk.calliope import base

DETAILED_HELP = {  # Dict[str, str]
    'brief': 'Create, list, describe and delete Compute Engine snapshot groups',
}


# --- Alpha Group Definition ---
@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@base.UniverseCompatible
class SnapshotGroupsAlpha(base.Group):
  """Create, list, describe and delete Compute Engine snapshot groups."""

  category = base.INSTANCES_CATEGORY
  detailed_help = DETAILED_HELP.copy()


# --- Beta Group Definition ---
@base.Hidden  # This hides the BETA group from public documentation
@base.ReleaseTracks(base.ReleaseTrack.BETA)
@base.UniverseCompatible
class SnapshotGroupsBeta(base.Group):
  """Create, list, describe and delete Compute Engine snapshot groups."""

  category = base.INSTANCES_CATEGORY
  detailed_help = DETAILED_HELP.copy()



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Command to download the server connect configuration."""

from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core.credentials import transports
from googlecloudsdk.core.util import files


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@base.DefaultUniverseOnly
class DownloadServerConnectConfig(base.Command):
  """Download the server connect configuration for a Network Monitoring Provider."""

  detailed_help = {
      'BRIEF': 'Download the server connect configuration.',
      'DESCRIPTION': (
          """\
          Downloads the server connect configuration for a given Network Monitoring Provider.
          The response is a zip file.
          """
      ),
      'EXAMPLES': (
          """\
          To download the server connect configuration for provider `my-provider`, run:

            $ {command} --network-monitoring-provider=my-provider --location=global --output-file=config.zip
          """
      ),
  }

  @staticmethod
  def Args(parser):
    parser.add_argument(
        '--network-monitoring-provider',
        required=True,
        help='The ID of the Network Monitoring Provider.',
    )
    parser.add_argument(
        '--location',
        required=True,
        help=(
            'The location of the Network Monitoring Provider (example:'
            ' `global`).'
        ),
    )
    parser.add_argument(
        '--output-file',
        required=True,
        help='The path to save the downloaded server connect configuration.',
    )

  def Run(self, args):
    project = properties.VALUES.core.project.Get(required=True)
    location = args.location
    provider_id = args.network_monitoring_provider
    release_track = self.ReleaseTrack()
    if release_track == base.ReleaseTrack.ALPHA:
      api_version = 'v1alpha1'
    else:
      api_version = 'v1'

    parent = f'projects/{project}/locations/{location}/networkMonitoringProviders/{provider_id}'
    request_path = (
        f'{api_version}/{parent}/monitoringPoints:downloadServerConnectConfig'
    )

    base_uri = apis.GetEffectiveApiEndpoint('networkmanagement', api_version)
    uri = f'{base_uri}{request_path}'
    http = transports.GetApitoolsTransport(response_encoding=None)

    response, body = http.request(
        uri,
        method='GET',
        headers={},
    )

    if response.status != 200:
      raise exceptions.HttpException(
          f'API request failed with status {response.status}:'
          f' {body.decode("utf-8")}'
      )

    content_type = (
        response['content-type'] if 'content-type' in response else 'unknown'
    )

    try:
      with files.BinaryFileWriter(args.output_file) as f:
        f.write(body)
      log.status.Print(
          f'Downloaded {content_type} server connect configuration to'
          f' [{args.output_file}]'
      )
    except Exception as e:
      raise exceptions.BadFileException(
          f'Failed to write file [{args.output_file}]: {e}'
      )

    return None


@base.ReleaseTracks(base.ReleaseTrack.GA)
@base.DefaultUniverseOnly
class DownloadServerConnectConfigGa(DownloadServerConnectConfig):
  """Download the server connect configuration."""

  pass



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Upload workflow using gRPC bidi streaming API client."""

from __future__ import annotations

import abc
import collections
import io
import os

from googlecloudsdk.api_lib.storage import cloud_api
from googlecloudsdk.api_lib.storage import errors as api_errors
from googlecloudsdk.api_lib.storage import request_config_factory
from googlecloudsdk.api_lib.storage.gcs_grpc import grpc_util
from googlecloudsdk.api_lib.storage.gcs_grpc import metadata_util
from googlecloudsdk.api_lib.storage.gcs_grpc import retry_util
from googlecloudsdk.api_lib.storage.gcs_grpc_bidi_streaming import retry_util as bidi_retry_util
from googlecloudsdk.command_lib.storage import encryption_util
from googlecloudsdk.command_lib.storage import errors as command_errors
from googlecloudsdk.command_lib.storage import fast_crc32c_util
from googlecloudsdk.command_lib.storage import gzip_util
from googlecloudsdk.command_lib.storage import hash_util
from googlecloudsdk.command_lib.storage import posix_util
from googlecloudsdk.command_lib.storage.resources import resource_reference
from googlecloudsdk.command_lib.storage.tasks.cp import copy_util
from googlecloudsdk.command_lib.util import crc32c
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
import six


# TODO: b/441010615 - Remove this constant once the flush size is configurable
# by the user. The default flush size is 50MiB.
_DEFAULT_FLUSH_SIZE = 50 * 1024 * 1024


def _should_perform_on_fly_checksumming(
    request_config: request_config_factory._GcsRequestConfig,
    source_resource: (
        resource_reference.FileObjectResource
        | resource_reference.ObjectResource
        | None
    ),
) -> bool:
  """Returns whether to perform on-the-fly checksumming.

  Args:
    request_config: Tracks additional request preferences.
    source_resource: Contains the source StorageUrl and source object
      metadata.

  Returns:
    True if on-the-fly checksumming should be performed, False otherwise.
  """
  if (
      properties.VALUES.storage.check_hashes.Get()
      == properties.CheckHashes.NEVER.value
  ):
    return False

  # Do not perform on-the-fly checksumming for cloud resources, in case of daisy
  # chaining. As gcloud_crc32c lacks support for computing hashes for data
  # chunks.
  # TODO: b/479490828 - Add support for computing hashes for data chunks.
  if isinstance(source_resource, resource_reference.CloudResource):
    return False

  # On-the-fly checksumming is disabled for gzip and symlinks because the data
  # being transferred is different from the source file's content. For gzip, the
  # content is compressed, and for symlinks, the content is the link's path.

  source_path = (
      source_resource.storage_url.versionless_url_string
      if source_resource is not None
      else None
  )
  # TODO: b/480766439 - Enable on-the-fly checksumming for gzip.
  if gzip_util.should_gzip_locally(request_config.gzip_settings, source_path):
    return False
  # TODO: b/466078602 - Enable on-the-fly checksumming for symlinks.
  if (
      isinstance(source_resource, resource_reference.FileObjectResource)
      and source_resource.is_symlink
      and request_config.preserve_symlinks
  ):
    return False
  # On fly checksumming is performed only when fast_crc32c is available.
  return fast_crc32c_util.check_if_will_use_fast_crc32c(install_if_missing=True)


class _Upload(six.with_metaclass(abc.ABCMeta, object)):
  """Base class shared by different upload strategies.

  This class is not thread-safe due to the state maintained in instance
  variables like `_uploaded_so_far`, `_buffer`, and `_initial_request`.
  """

  def __init__(
      self,
      client,
      source_stream: io.IOBase,
      destination_resource: (
          resource_reference.ObjectResource | resource_reference.UnknownResource
      ),
      request_config: request_config_factory._GcsRequestConfig,
      source_resource: (
          resource_reference.FileObjectResource
          | resource_reference.ObjectResource
          | None
      ),
      start_offset: int = 0,
      delegator: cloud_api.CloudApi | None = None,
      posix_to_set: posix_util.PosixAttributes | None = None,
      encryption_key: encryption_util.EncryptionKey | None = None,
      is_streaming_upload: bool = False,
  ):
    """Initializes _Upload.

    Args:
      client (gapic_clients.storage_v2.services.storage.client.StorageClient):
        The GAPIC client.
      source_stream: Yields bytes to upload.
      destination_resource: Metadata for the destination object.
      request_config: Tracks additional request preferences.
      source_resource: Contains the source StorageUrl and source object
        metadata.
      start_offset: The offset from the beginning of the object at which the
        data should be written.
      delegator: The client used to make non-bidi streaming or metadata API
        calls.
      posix_to_set: POSIX attributes to set on the destination object.
      encryption_key: The encryption key to use for the upload.
      is_streaming_upload: Whether the upload is streaming.
    """
    self._client = client
    self._source_stream = source_stream
    self._destination_resource = destination_resource
    self._request_config = request_config
    self._start_offset = start_offset
    # Maintain the state of upload. Useful for resumable and streaming uploads.
    self._uploaded_so_far = start_offset
    self._source_resource = source_resource
    self._delegator = delegator
    self._should_use_crc32c = _should_perform_on_fly_checksumming(
        self._request_config, self._source_resource
    )
    self._buffer = collections.deque(maxlen=self._get_max_buffer_size())
    self._initial_request = None
    self._should_finalize_writes = (
        properties.VALUES.storage.bidi_streaming_finalize_writes.GetBool()
    )
    self._finalized_resource = None
    self._redirection_handler = bidi_retry_util.BidiRedirectedTokenErrorHandler(
        self._client,
        source_resource=self._source_resource,
        destination_resource=self._destination_resource,
    )
    self._posix_to_set = posix_to_set
    self._encryption_key = encryption_key
    self._encryption_params = grpc_util.get_encryption_request_params(
        self._client, encryption_key
    )
    self._is_streaming_upload = is_streaming_upload

  def _get_max_buffer_size(self):
    """Returns the maximum buffer size."""
    # The buffer size is calculated as follows:
    # 1. The default flush size is 50MiB.
    # 2. The maximum size of a single request is 2MiB.
    # 4. The maximum size of the buffer is calculated as:
    #    (50MiB + (2MiB)) // (2MiB) = 26
    # 5. Similarily for buffer size of odd flush size:
    #    (51MiB + (2MiB)) // (2MiB) = 26
    # 6. The maximum number of requests that can be buffered is 13.
    chunk_size = (
        self._client.types.ServiceConstants.Values.MAX_WRITE_CHUNK_BYTES
    )
    return (_DEFAULT_FLUSH_SIZE + (chunk_size)) // (chunk_size)

  def _initialize_bidi_write(self):
    """Initializes the generator for the upload."""
    self._uploaded_so_far = self._start_offset
    self._source_stream.seek(self._start_offset, os.SEEK_SET)

  def _get_crc32c_hash(self, data: bytes, length: int) -> int | None:
    """Returns CRC32C hash of the given data."""
    if not self._should_use_crc32c:
      return None
    crc32c_hash = hash_util.get_hash_from_data_chunk_or_file(
        self._source_resource.storage_url.resource_name if self._source_resource
        else None,
        data,
        hash_util.HashAlgorithm.CRC32C,
        self._uploaded_so_far,
        self._uploaded_so_far + length,
        is_streaming=self._is_streaming_upload,
    )
    return int.from_bytes(crc32c_hash.digest(), byteorder='big')

  def _send_buffer_to_bidi_write_rpc(self, bidi_write_rpc) -> None:
    """Sends the buffer to the bidi write RPC."""
    if not self._buffer:
      return

    for request in self._buffer:
      bidi_write_rpc.send(request)
    while True:
      response = bidi_write_rpc.recv()
      if response.write_handle:
        self._initial_request.append_object_spec.write_handle = (
            response.write_handle
        )
      if (
          response.persisted_size is not None
          and response.persisted_size == self._uploaded_so_far
      ):
        self._start_offset = response.persisted_size
        break
      elif response.resource:
        self._finalized_resource = (
            metadata_util.get_object_resource_from_grpc_object(
                response.resource
            )
        )
        break

    self._buffer.clear()

  def _half_close_bidi_write_rpc(self, bidi_write_rpc) -> None:
    """Performs a client-side half-close of the bidi write RPC stream.

    This is needed when `_should_finalize_writes` is false to signal the server
    that the client is done sending requests. It also processes any remaining
    server responses. The full RPC stream closure(cancel) is handled by
    `bidi_write_rpc.close()`.

    Args:
      bidi_write_rpc: The bidi write RPC object.
    """
    if self._should_finalize_writes:
      return
    # Signal the server to half-close the stream.
    bidi_write_rpc.requests_done()
    while True:
      try:
        response = bidi_write_rpc.recv()
      except StopIteration:
        break
      if response.persisted_size == self._uploaded_so_far:
        self._start_offset = response.persisted_size

  def _update_initial_request(self, response):
    """Updates the initial request for the bidi write RPC.

    If the initial request is an append_object_spec, then we update the
    write_handle in the initial request with the one from the response, for
    retries.
    If the initial request is a write_object_spec, then we need to update the
    initial request with an append_object_spec based on the response. This is
    because write_object_spec is only used for creating an appendable object.
    For retrying on failures, we need to use append_object_spec to append to an
    existing object.

    For failures on the first request(creating or appending),
    this method is not called, and the initial request is not modified.

    Args:
      response (gapic_clients.storage_v2.types.BidiWriteObjectResponse): The
        response from the bidi write RPC.
    """
    if self._initial_request.append_object_spec:
      if response.write_handle:
        self._initial_request.append_object_spec.write_handle = (
            response.write_handle
        )
      return
    self._initial_request = self._client.types.BidiWriteObjectRequest(
        append_object_spec=self._client.types.AppendObjectSpec(
            bucket=response.resource.bucket,
            object=response.resource.name,
            generation=response.resource.generation,
            if_metageneration_match=(
                self._request_config.precondition_metageneration_match
            ),
            write_handle=response.write_handle,
        ),
    )

  @retry_util.grpc_default_retryer
  def _bidi_write_appendable_object(self, resuming_upload: bool = False):
    """Performs the bidi write RPC for an appendable object."""
    bidi_write_rpc = self._redirection_handler.start_bidi_rpc_with_retry_on_redirected_token_error(
        self._initial_request
    )
    try:
      response = bidi_write_rpc.recv()
      self._update_initial_request(response)

      if resuming_upload:
        self._check_existing_destination_is_valid(response.resource)
        self._start_offset = response.resource.size

      # If the upload is being retried, we need to send the existing buffer to
      # the bidi write RPC before initializing the upload. This is because the
      # buffer contains the data that was written before the upload was
      # interrupted. We will be retrying on the entire buffer.
      self._send_buffer_to_bidi_write_rpc(bidi_write_rpc)
      self._initialize_bidi_write()
      flush_interval = self._start_offset + _DEFAULT_FLUSH_SIZE

      while True:
        data = self._source_stream.read(
            self._client.types.ServiceConstants.Values.MAX_WRITE_CHUNK_BYTES
        )
        length_of_data = len(data)

        if self._uploaded_so_far + length_of_data >= flush_interval:
          should_flush = True
          flush_interval += _DEFAULT_FLUSH_SIZE
        else:
          should_flush = False

        finish_write = not length_of_data
        if finish_write:
          # Handles final request case.
          should_flush = True

        self._buffer.append(
            self._client.types.BidiWriteObjectRequest(
                write_offset=self._uploaded_so_far,
                checksummed_data=self._client.types.ChecksummedData(
                    content=data,
                    crc32c=self._get_crc32c_hash(data, length_of_data),
                ),
                flush=should_flush,
                state_lookup=should_flush,
                finish_write=(finish_write and self._should_finalize_writes),
            )
        )
        self._uploaded_so_far += length_of_data
        if should_flush:
          self._send_buffer_to_bidi_write_rpc(bidi_write_rpc)

        if finish_write:
          self._half_close_bidi_write_rpc(bidi_write_rpc)
          break
    except StopIteration as e:
      raise bidi_retry_util.BidiUploadStreamClosedError(
          'The BiDi upload stream was unexpectedly closed for object:'
          f' {self._destination_resource.storage_url.resource_name}'
      ) from e
    finally:
      # Cancel the rpc if it is still open.
      bidi_write_rpc.close()

  def _get_request_for_creating_append_object(self):
    """Returns the request for creating an appendable object.

    Returns:
      gapic_clients.storage_v2.types.BidiWriteObjectRequest: A
        BidiWriteObjectRequest instance.
    """
    destination_object = self._client.types.Object(
        name=self._destination_resource.storage_url.resource_name,
        bucket=grpc_util.get_full_bucket_name(
            self._destination_resource.storage_url.bucket_name
        ),
    )

    metadata_util.update_object_metadata_from_request_config(
        destination_object,
        self._request_config,
        self._source_resource,
        posix_to_set=self._posix_to_set,
    )

    metadata_util.set_metadata_if_source_is_object_resource(
        self._source_resource,
        destination_object,
    )

    write_object_spec = self._client.types.WriteObjectSpec(
        resource=destination_object,
        if_generation_match=copy_util.get_generation_match_value(
            self._request_config
        ),
        if_metageneration_match=(
            self._request_config.precondition_metageneration_match
        ),
        appendable=True,
        predefined_acl=self._request_config.predefined_acl_string,
    )
    return self._client.types.BidiWriteObjectRequest(
        write_object_spec=write_object_spec,
        common_object_request_params=self._encryption_params,
    )

  def _get_request_for_resuming_appendable_object_upload(
      self, destination_object: resource_reference.ObjectResource
  ):
    """Returns the request for resuming an appendable object upload."""
    append_object_spec = self._client.types.AppendObjectSpec(
        bucket=grpc_util.get_full_bucket_name(destination_object.bucket),
        object=destination_object.name,
        generation=destination_object.generation,
        if_metageneration_match=(
            self._request_config.precondition_metageneration_match
        ),
    )
    return self._client.types.BidiWriteObjectRequest(
        append_object_spec=append_object_spec,
        common_object_request_params=self._encryption_params,
    )

  def _check_existing_destination_is_valid(self, destination_object):
    """Checks if the existing destination object is valid."""
    if not self._should_use_crc32c:
      return
    crc32c_hash = hash_util.get_hash_from_file(
        self._source_resource.storage_url.resource_name,
        hash_util.HashAlgorithm.CRC32C,
        0,
        destination_object.size,
    )
    calculated_crc32c_hash = crc32c.get_hash(crc32c_hash)
    destination_crc32c_hash = crc32c.get_crc32c_hash_string_from_checksum(
        destination_object.checksums.crc32c
    )
    try:
      hash_util.validate_object_hashes_match(
          self._destination_resource.storage_url.url_string,
          calculated_crc32c_hash,
          destination_crc32c_hash,
      )
    except command_errors.HashMismatchError as e:
      self._delegator.delete_object(
          self._destination_resource.storage_url,
          request_config=self._request_config,
      )
      raise e
    else:
      log.info(
          'Destination object is valid. Resuming upload for object: %s',
          self._destination_resource.storage_url.resource_name,
      )

  @retry_util.grpc_default_retryer
  def _get_object_if_exists(self) -> resource_reference.ObjectResource | None:
    """Gets the destination object metadata if a resumable upload is possible.

    Returns:
      The destination object as a resource_reference.ObjectResource if it
      exists and is not finalized. Returns None if the object does not exist
      or if it is already finalized, as finalized objects must be overwritten
      rather than resumed.
    """
    try:
      object_resource = self._delegator.get_object_metadata(
          self._destination_resource.storage_url.bucket_name,
          self._destination_resource.storage_url.resource_name,
          self._request_config,
      )
    except api_errors.NotFoundError:
      log.debug(
          'Object %r does not exist. Proceeding with upload.',
          self._destination_resource.storage_url.resource_name,
      )
      return None
    if object_resource.metadata.timeFinalized is not None:
      log.debug(
          'Object %r is finalized. Proceeding with overwrite.',
          self._destination_resource.storage_url.resource_name,
      )
      return None
    return object_resource

  @retry_util.grpc_default_retryer
  def _get_object_metadata_after_upload(
      self,
  ) -> resource_reference.ObjectResource:
    """Returns the object metadata after the upload."""
    # For finalized objects, the object metadata is returned in the response of
    # the last request(i.e. the request with finish_write set).
    if self._finalized_resource:
      return self._finalized_resource
    read_request = self._client.types.BidiReadObjectRequest(
        read_object_spec=self._client.types.BidiReadObjectSpec(
            bucket=self._initial_request.append_object_spec.bucket,
            object=self._initial_request.append_object_spec.object,
            generation=self._initial_request.append_object_spec.generation,
            common_object_request_params=self._encryption_params,
        )
    )
    bidi_rpc = self._redirection_handler.start_bidi_rpc_with_retry_on_redirected_token_error(
        read_request
    )
    try:
      response = bidi_rpc.recv()
    finally:
      bidi_rpc.close()
    return metadata_util.get_object_resource_from_grpc_object(response.metadata)

  @abc.abstractmethod
  def run(self):
    """Performs an upload and returns an Object message."""
    raise NotImplementedError()


class SimpleUpload(_Upload):
  """Uploads an object in single-shot mode."""

  def run(self):
    """Uploads the object in single-shot mode.

    Returns:
      resource_reference.ObjectResource with object metadata.

    Raises:
      CloudApiError: API returned an error.
    """
    self._initial_request = self._get_request_for_creating_append_object()

    self._bidi_write_appendable_object()

    return self._get_object_metadata_after_upload()


class ResumableUpload(_Upload):
  """Uploads an object in resumable mode."""

  def run(self):
    """Uploads the object in resumable mode.

    Returns:
      resource_reference.ObjectResource with object metadata.

    Raises:
      CloudApiError: API returned an error.
    """
    destination_object = self._get_object_if_exists()
    if destination_object:
      self._initial_request = (
          self._get_request_for_resuming_appendable_object_upload(
              destination_object
          )
      )
      self._start_offset = destination_object.size
      log.info(
          'Attempting to resume upload for object: %s at offset: %s',
          destination_object.name,
          destination_object.size,
      )
      try:
        self._bidi_write_appendable_object(resuming_upload=True)
        return self._get_object_metadata_after_upload()
      except command_errors.HashMismatchError as e:
        log.info(
            'Failed to validate checksum for object: %s with error: %s.'
            'Deleting the existing object and retrying the upload.',
            destination_object.name,
            e,
        )

    return SimpleUpload(
        client=self._client,
        source_stream=self._source_stream,
        destination_resource=self._destination_resource,
        request_config=self._request_config,
        source_resource=self._source_resource,
        start_offset=0,
        delegator=self._delegator,
        posix_to_set=self._posix_to_set,
        encryption_key=self._encryption_key,
    ).run()



# -*- coding: utf-8 -*- #
# Copyright 2020 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for updating security policies."""


import typing

from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute.security_policies import client
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.compute import scope as compute_scope
from googlecloudsdk.command_lib.compute.security_policies import flags
from googlecloudsdk.command_lib.compute.security_policies import security_policies_utils


@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.PREVIEW)
@base.DefaultUniverseOnly
class UpdateGa(base.UpdateCommand):
  """Update a Compute Engine security policy.

  *{command}* is used to update security policies.

  ## EXAMPLES

  To update the description run this:

    $ {command} SECURITY_POLICY --description='new description'
  """

  SECURITY_POLICY_ARG = None

  @classmethod
  def Args(cls, parser):
    cls.SECURITY_POLICY_ARG = flags.SecurityPolicyMultiScopeArgument()
    cls.SECURITY_POLICY_ARG.AddArgument(parser, operation_type='update')
    parser.add_argument(
        '--description',
        help=('An optional, textual description for the security policy.'))

    flags.AddCloudArmorAdaptiveProtection(parser)
    flags.AddAdvancedOptions(parser)
    flags.AddRecaptchaOptions(parser)
    flags.AddDdosProtectionConfigWithAdvancedPreview(parser)
    flags.AddNetworkDdosAdaptiveProtection(parser)
    flags.AddNetworkDdosImpactedBaselineThreshold(parser)

  def _ValidateArgs(self, args):
    """Validates that at least one field to update is specified.

    Args:
      args: The arguments given to the update command.
    """

    if not (
        args.IsSpecified('description')
        or args.IsSpecified('enable_layer7_ddos_defense')
        or args.IsSpecified('layer7_ddos_defense_rule_visibility')
        or args.IsSpecified('json_parsing')
        or args.IsSpecified('json_custom_content_types')
        or args.IsSpecified('log_level')
        or args.IsSpecified('recaptcha_redirect_site_key')
        or args.IsSpecified('network_ddos_protection')
        or args.IsSpecified('network_ddos_adaptive_protection')
        or args.IsSpecified('network_ddos_impacted_baseline_threshold')
        or args.IsSpecified('clear_network_ddos_impacted_baseline_threshold')
        or args.IsSpecified('user_ip_request_headers')
    ):
      parameter_names = [
          '--description',
          '--enable-layer7-ddos-defense',
          '--layer7-ddos-defense-rule-visibility',
          '--json-parsing',
          '--json-custom-content-types',
          '--log-level',
          '--recaptcha-redirect-site-key',
          '--network-ddos-protection',
          '--network-ddos-adaptive-protection',
          '--network-ddos-impacted-baseline-threshold',
          '--clear-network-ddos-impacted-baseline-threshold',
          '--user-ip-request-headers',
      ]
      raise exceptions.MinimumArgumentException(
          parameter_names, 'Please specify at least one property to update')

  def Run(self, args):
    self._ValidateArgs(args)
    field_mask = []

    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    assert self.SECURITY_POLICY_ARG is not None
    ref = self.SECURITY_POLICY_ARG.ResolveAsResource(
        args, holder.resources, default_scope=compute_scope.ScopeEnum.GLOBAL)
    security_policy = client.SecurityPolicy(
        ref=ref, compute_client=holder.client)
    existing_security_policy = typing.cast(
        typing.Any, security_policy.Describe()
    )[0]
    description = existing_security_policy.description
    adaptive_protection_config = (
        existing_security_policy.adaptiveProtectionConfig)
    advanced_options_config = existing_security_policy.advancedOptionsConfig
    recaptcha_options_config = existing_security_policy.recaptchaOptionsConfig
    ddos_protection_config = existing_security_policy.ddosProtectionConfig

    if args.description is not None:
      description = args.description
      field_mask.append('description')
    if (args.IsSpecified('enable_layer7_ddos_defense') or
        args.IsSpecified('layer7_ddos_defense_rule_visibility')):
      adaptive_protection_config = (
          security_policies_utils.CreateAdaptiveProtectionConfig(
              holder.client, args, adaptive_protection_config))
      field_mask.append('adaptive_protection_config')
    if (args.IsSpecified('json_parsing') or
        args.IsSpecified('json_custom_content_types') or
        args.IsSpecified('log_level') or
        args.IsSpecified('user_ip_request_headers')):
      advanced_options_config = (
          security_policies_utils.CreateAdvancedOptionsConfig(
              holder.client,
              args,
              advanced_options_config,
              enable_large_body_size=False,
          )
      )
      field_mask.append('advanced_options_config')
    if args.IsSpecified('recaptcha_redirect_site_key'):
      recaptcha_options_config = (
          security_policies_utils.CreateRecaptchaOptionsConfig(
              holder.client, args, recaptcha_options_config))
      field_mask.append('recaptcha_options_config')
    if args.IsSpecified('network_ddos_protection'):
      ddos_protection_config = (
          security_policies_utils.CreateDdosProtectionConfig(
              holder.client, args, ddos_protection_config))
      field_mask.append('ddos_protection_config')
    if args.IsSpecified('network_ddos_adaptive_protection'):
      ddos_protection_config = security_policies_utils.CreateDdosProtectionConfigWithDdosAdaptiveProtection(
          holder.client, args, ddos_protection_config
      )
      if 'ddos_protection_config' not in field_mask:
        field_mask.append('ddos_protection_config')
    if args.IsSpecified('network_ddos_impacted_baseline_threshold'):
      ddos_protection_config = security_policies_utils.CreateDdosProtectionConfigWithNetworkDdosImpactedBaselineThreshold(
          holder.client, args, ddos_protection_config
      )
      if 'ddos_protection_config' not in field_mask:
        field_mask.append('ddos_protection_config')
      field_mask.append(
          'ddos_protection_config.ddos_impacted_baseline_threshold'
      )
    elif args.IsSpecified('clear_network_ddos_impacted_baseline_threshold'):
      if ddos_protection_config is None:
        ddos_protection_config = (
            holder.client.messages.SecurityPolicyDdosProtectionConfig()
        )
      ddos_protection_config.ddosImpactedBaselineThreshold = None
      if 'ddos_protection_config' not in field_mask:
        field_mask.append('ddos_protection_config')
      field_mask.append(
          'ddos_protection_config.ddos_impacted_baseline_threshold'
      )

    updated_security_policy = holder.client.messages.SecurityPolicy(
        description=description,
        adaptiveProtectionConfig=adaptive_protection_config,
        advancedOptionsConfig=advanced_options_config,
        recaptchaOptionsConfig=recaptcha_options_config,
        ddosProtectionConfig=ddos_protection_config,
        fingerprint=existing_security_policy.fingerprint)

    return security_policy.Patch(
        security_policy=updated_security_policy,
        field_mask=','.join(field_mask))


@base.ReleaseTracks(base.ReleaseTrack.BETA)
@base.DefaultUniverseOnly
class UpdateBeta(UpdateGa):
  """Update a Compute Engine security policy.

  *{command}* is used to update security policies.

  ## EXAMPLES

  To update the description run this:

    $ {command} SECURITY_POLICY --description='new description'
  """

  SECURITY_POLICY_ARG = None

  @classmethod
  def Args(cls, parser):
    cls.SECURITY_POLICY_ARG = flags.SecurityPolicyMultiScopeArgument()
    cls.SECURITY_POLICY_ARG.AddArgument(parser, operation_type='update')
    parser.add_argument(
        '--description',
        help=('An optional, textual description for the security policy.'))

    flags.AddCloudArmorAdaptiveProtection(parser)
    flags.AddCloudArmorAdaptiveProtectionAutoDeploy(parser)
    flags.AddAdvancedOptions(parser, enable_large_body_size=True)
    flags.AddRecaptchaOptions(parser)
    flags.AddDdosProtectionConfigWithAdvancedPreview(parser)
    flags.AddNetworkDdosAdaptiveProtection(parser)
    flags.AddNetworkDdosImpactedBaselineThreshold(parser)

  def _ValidateArgs(self, args):
    """Validates that at least one field to update is specified.

    Args:
      args: The arguments given to the update command.
    """
    if not (
        args.IsSpecified('description')
        or args.IsSpecified('enable_layer7_ddos_defense')
        or args.IsSpecified('layer7_ddos_defense_rule_visibility')
        or args.IsSpecified('json_parsing')
        or args.IsSpecified('json_custom_content_types')
        or args.IsSpecified('log_level')
        or args.IsSpecified('request_body_inspection_size')
        or args.IsSpecified('user_ip_request_headers')
        or args.IsSpecified('recaptcha_redirect_site_key')
        or args.IsSpecified('network_ddos_protection')
        or args.IsSpecified('layer7_ddos_defense_auto_deploy_load_threshold')
        or args.IsSpecified(
            'layer7_ddos_defense_auto_deploy_confidence_threshold'
        )
        or args.IsSpecified(
            'layer7_ddos_defense_auto_deploy_impacted_baseline_threshold'
        )
        or args.IsSpecified('layer7_ddos_defense_auto_deploy_expiration_sec')
        or args.IsSpecified('network_ddos_adaptive_protection')
        or args.IsSpecified('network_ddos_impacted_baseline_threshold')
        or args.IsSpecified('clear_network_ddos_impacted_baseline_threshold')
    ):
      parameter_names = [
          '--description',
          '--enable-layer7-ddos-defense',
          '--layer7-ddos-defense-rule-visibility',
          '--json-parsing',
          '--json-custom-content-types',
          '--log-level',
          '--user-ip-request-headers',
          '--request-body-inspection-size',
          '--recaptcha-redirect-site-key',
          '--network-ddos-protection',
          '--layer7-ddos-defense-auto-deploy-load-threshold',
          '--layer7-ddos-defense-auto-deploy-confidence-threshold',
          '--layer7-ddos-defense-auto-deploy-impacted-baseline-threshold',
          '--layer7-ddos-defense-auto-deploy-expiration-sec',
          '--network-ddos-adaptive-protection',
          '--network-ddos-impacted-baseline-threshold',
          '--clear-network-ddos-impacted-baseline-threshold',
      ]
      raise exceptions.MinimumArgumentException(
          parameter_names, 'Please specify at least one property to update')

  def Run(self, args):
    self._ValidateArgs(args)
    field_mask = []

    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    assert self.SECURITY_POLICY_ARG is not None
    ref = self.SECURITY_POLICY_ARG.ResolveAsResource(
        args, holder.resources, default_scope=compute_scope.ScopeEnum.GLOBAL
    )
    security_policy = client.SecurityPolicy(
        ref=ref, compute_client=holder.client)
    existing_security_policy = typing.cast(
        typing.Any, security_policy.Describe()
    )[0]
    description = existing_security_policy.description
    adaptive_protection_config = (
        existing_security_policy.adaptiveProtectionConfig)
    advanced_options_config = existing_security_policy.advancedOptionsConfig
    recaptcha_options_config = existing_security_policy.recaptchaOptionsConfig
    ddos_protection_config = existing_security_policy.ddosProtectionConfig

    if args.description is not None:
      description = args.description
      field_mask.append('description')
    if (args.IsSpecified('enable_layer7_ddos_defense') or
        args.IsSpecified('layer7_ddos_defense_rule_visibility') or
        args.IsSpecified('layer7_ddos_defense_auto_deploy_load_threshold') or
        args.IsSpecified('layer7_ddos_defense_auto_deploy_confidence_threshold')
        or args.IsSpecified(
            'layer7_ddos_defense_auto_deploy_impacted_baseline_threshold') or
        args.IsSpecified('layer7_ddos_defense_auto_deploy_expiration_sec')):
      adaptive_protection_config = (
          security_policies_utils
          .CreateAdaptiveProtectionConfigWithAutoDeployConfig(
              holder.client, args, adaptive_protection_config))
      field_mask.append('adaptive_protection_config')
    if (
        args.IsSpecified('json_parsing')
        or args.IsSpecified('json_custom_content_types')
        or args.IsSpecified('log_level')
        or args.IsSpecified('request_body_inspection_size')
        or args.IsSpecified('user_ip_request_headers')
    ):
      advanced_options_config = (
          security_policies_utils.CreateAdvancedOptionsConfig(
              holder.client,
              args,
              advanced_options_config,
              enable_large_body_size=True,
          )
      )
      field_mask.append('advanced_options_config')
    if args.IsSpecified('recaptcha_redirect_site_key'):
      recaptcha_options_config = (
          security_policies_utils.CreateRecaptchaOptionsConfig(
              holder.client, args, recaptcha_options_config))
      field_mask.append('recaptcha_options_config')
    if args.IsSpecified('network_ddos_protection'):
      ddos_protection_config = (
          security_policies_utils.CreateDdosProtectionConfig(
              holder.client, args, ddos_protection_config))
      field_mask.append('ddos_protection_config')
    if args.IsSpecified('network_ddos_adaptive_protection'):
      ddos_protection_config = security_policies_utils.CreateDdosProtectionConfigWithDdosAdaptiveProtection(
          holder.client, args, ddos_protection_config
      )
      if 'ddos_protection_config' not in field_mask:
        field_mask.append('ddos_protection_config')
    if args.IsSpecified('network_ddos_impacted_baseline_threshold'):
      ddos_protection_config = security_policies_utils.CreateDdosProtectionConfigWithNetworkDdosImpactedBaselineThreshold(
          holder.client, args, ddos_protection_config
      )
      if 'ddos_protection_config' not in field_mask:
        field_mask.append('ddos_protection_config')
      field_mask.append(
          'ddos_protection_config.ddos_impacted_baseline_threshold'
      )
    elif args.IsSpecified('clear_network_ddos_impacted_baseline_threshold'):
      if ddos_protection_config is None:
        ddos_protection_config = (
            holder.client.messages.SecurityPolicyDdosProtectionConfig()
        )
      ddos_protection_config.ddosImpactedBaselineThreshold = None
      if 'ddos_protection_config' not in field_mask:
        field_mask.append('ddos_protection_config')
      field_mask.append(
          'ddos_protection_config.ddos_impacted_baseline_threshold'
      )

    updated_security_policy = holder.client.messages.SecurityPolicy(
        description=description,
        adaptiveProtectionConfig=adaptive_protection_config,
        advancedOptionsConfig=advanced_options_config,
        recaptchaOptionsConfig=recaptcha_options_config,
        ddosProtectionConfig=ddos_protection_config,
        fingerprint=existing_security_policy.fingerprint)

    return security_policy.Patch(
        security_policy=updated_security_policy,
        field_mask=','.join(field_mask))


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@base.DefaultUniverseOnly
class UpdateAlpha(UpdateBeta):
  """Update a Compute Engine security policy.

  *{command}* is used to update security policies.

  ## EXAMPLES

  To update the description run this:

    $ {command} SECURITY_POLICY --description='new description'
  """

  SECURITY_POLICY_ARG = None

  @classmethod
  def Args(cls, parser):
    cls.SECURITY_POLICY_ARG = flags.SecurityPolicyMultiScopeArgument()
    cls.SECURITY_POLICY_ARG.AddArgument(parser, operation_type='update')
    parser.add_argument(
        '--description',
        help=('An optional, textual description for the security policy.'))

    flags.AddCloudArmorAdaptiveProtection(parser)
    flags.AddCloudArmorAdaptiveProtectionAutoDeploy(parser)
    flags.AddAdvancedOptions(parser, enable_large_body_size=True)
    flags.AddRecaptchaOptions(parser)
    flags.AddDdosProtectionConfigWithAdvancedPreview(parser)
    flags.AddDdosProtectionConfigOld(parser)
    flags.AddNetworkDdosAdaptiveProtection(parser)
    flags.AddNetworkDdosImpactedBaselineThreshold(parser)

    parser.add_argument(
        '--enable-ml',
        action='store_true',
        default=None,
        help=('Whether to enable Cloud Armor Adaptive Protection'))

  def _ValidateArgs(self, args):
    """Validates that at least one field to update is specified.

    Args:
      args: The arguments given to the update command.
    """

    if not (
        args.IsSpecified('description')
        or args.IsSpecified('enable_ml')
        or args.IsSpecified('enable_layer7_ddos_defense')
        or args.IsSpecified('layer7_ddos_defense_rule_visibility')
        or args.IsSpecified('json_parsing')
        or args.IsSpecified('json_custom_content_types')
        or args.IsSpecified('log_level')
        or args.IsSpecified('request_body_inspection_size')
        or args.IsSpecified('user_ip_request_headers')
        or args.IsSpecified('recaptcha_redirect_site_key')
        or args.IsSpecified('network_ddos_protection')
        or args.IsSpecified('network_ddos_adaptive_protection')
        or args.IsSpecified('network_ddos_impacted_baseline_threshold')
        or args.IsSpecified('clear_network_ddos_impacted_baseline_threshold')
        or args.IsSpecified('ddos_protection')
    ):
      parameter_names = [
          '--description',
          '--enable-ml',
          '--enable-layer7-ddos-defense',
          '--layer7-ddos-defense-rule-visibility',
          '--json-parsing',
          '--json-custom-content-types',
          '--log-level',
          '--request-body-inspection-size',
          '--user-ip-request-headers',
          '--recaptcha-redirect-site-key',
          '--network-ddos-protection',
          '--network-ddos-adaptive-protection',
          '--network-ddos-impacted-baseline-threshold',
          '--clear-network-ddos-impacted-baseline-threshold',
          '--ddos-protection',
      ]
      raise exceptions.MinimumArgumentException(
          parameter_names, 'Please specify at least one property to update')

  def Run(self, args):
    self._ValidateArgs(args)
    field_mask = []

    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    assert self.SECURITY_POLICY_ARG is not None
    ref = self.SECURITY_POLICY_ARG.ResolveAsResource(
        args, holder.resources, default_scope=compute_scope.ScopeEnum.GLOBAL
    )
    security_policy = client.SecurityPolicy(
        ref=ref, compute_client=holder.client)
    existing_security_policy = typing.cast(
        typing.Any, security_policy.Describe()
    )[0]
    description = existing_security_policy.description
    cloud_armor_config = existing_security_policy.cloudArmorConfig
    adaptive_protection_config = (
        existing_security_policy.adaptiveProtectionConfig)
    advanced_options_config = existing_security_policy.advancedOptionsConfig
    recaptcha_options_config = existing_security_policy.recaptchaOptionsConfig
    ddos_protection_config = existing_security_policy.ddosProtectionConfig

    if args.description is not None:
      description = args.description
      field_mask.append('description')
    if args.enable_ml is not None:
      cloud_armor_config = security_policies_utils.CreateCloudArmorConfig(
          holder.client, args)
      field_mask.append('cloud_armor_config')
    if (args.IsSpecified('enable_layer7_ddos_defense') or
        args.IsSpecified('layer7_ddos_defense_rule_visibility') or
        args.IsSpecified('layer7_ddos_defense_auto_deploy_load_threshold') or
        args.IsSpecified('layer7_ddos_defense_auto_deploy_confidence_threshold')
        or args.IsSpecified(
            'layer7_ddos_defense_auto_deploy_impacted_baseline_threshold') or
        args.IsSpecified('layer7_ddos_defense_auto_deploy_expiration_sec')):
      adaptive_protection_config = (
          security_policies_utils
          .CreateAdaptiveProtectionConfigWithAutoDeployConfig(
              holder.client, args, adaptive_protection_config))
      field_mask.append('adaptive_protection_config')
    if (
        args.IsSpecified('json_parsing')
        or args.IsSpecified('json_custom_content_types')
        or args.IsSpecified('log_level')
        or args.IsSpecified('request_body_inspection_size')
        or args.IsSpecified('user_ip_request_headers')
    ):
      advanced_options_config = (
          security_policies_utils.CreateAdvancedOptionsConfig(
              holder.client,
              args,
              advanced_options_config,
              enable_large_body_size=True,
          )
      )
      field_mask.append('advanced_options_config')
    if args.IsSpecified('recaptcha_redirect_site_key'):
      recaptcha_options_config = (
          security_policies_utils.CreateRecaptchaOptionsConfig(
              holder.client, args, recaptcha_options_config))
      field_mask.append('recaptcha_options_config')
    if args.IsSpecified('ddos_protection'):
      ddos_protection_config = (
          security_policies_utils.CreateDdosProtectionConfigOld(
              holder.client, args, ddos_protection_config))
      if 'ddos_protection_config' not in field_mask:
        field_mask.append('ddos_protection_config')
    if args.IsSpecified('network_ddos_protection'):
      ddos_protection_config = (
          security_policies_utils.CreateDdosProtectionConfig(
              holder.client, args, ddos_protection_config))
      if 'ddos_protection_config' not in field_mask:
        field_mask.append('ddos_protection_config')
    if args.IsSpecified('network_ddos_adaptive_protection'):
      ddos_protection_config = security_policies_utils.CreateDdosProtectionConfigWithDdosAdaptiveProtection(
          holder.client, args, ddos_protection_config
      )
      if 'ddos_protection_config' not in field_mask:
        field_mask.append('ddos_protection_config')
    if args.IsSpecified('network_ddos_impacted_baseline_threshold'):
      ddos_protection_config = security_policies_utils.CreateDdosProtectionConfigWithNetworkDdosImpactedBaselineThreshold(
          holder.client, args, ddos_protection_config
      )
      if 'ddos_protection_config' not in field_mask:
        field_mask.append('ddos_protection_config')
      field_mask.append(
          'ddos_protection_config.ddos_impacted_baseline_threshold'
      )
    elif args.IsSpecified('clear_network_ddos_impacted_baseline_threshold'):
      if ddos_protection_config is None:
        ddos_protection_config = (
            holder.client.messages.SecurityPolicyDdosProtectionConfig()
        )
      ddos_protection_config.ddosImpactedBaselineThreshold = None
      if 'ddos_protection_config' not in field_mask:
        field_mask.append('ddos_protection_config')
      field_mask.append(
          'ddos_protection_config.ddos_impacted_baseline_threshold'
      )

    updated_security_policy = holder.client.messages.SecurityPolicy(
        description=description,
        cloudArmorConfig=cloud_armor_config,
        adaptiveProtectionConfig=adaptive_protection_config,
        advancedOptionsConfig=advanced_options_config,
        recaptchaOptionsConfig=recaptcha_options_config,
        ddosProtectionConfig=ddos_protection_config,
        fingerprint=existing_security_policy.fingerprint)

    return security_policy.Patch(
        security_policy=updated_security_policy,
        field_mask=','.join(field_mask))



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""List endpoint verdict change requests command."""

from googlecloudsdk.api_lib.network_security.firewall_endpoints import activation_api
from googlecloudsdk.api_lib.network_security.firewall_endpoints import wildfire_verdict_change_requests_api
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope.concepts import multitype
from googlecloudsdk.command_lib.network_security import activation_flags
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.command_lib.util.concepts import presentation_specs


@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
class List(base.ListCommand):
  """List WildFire Verdict Change Requests."""

  detailed_help = {
      'EXAMPLES': """\
        To list WildFire Verdict Change Requests for endpoint with full resource name `organizations/1234/locations/us-central1-a/firewallEndpoints/my-endpoint`, run:

          $ {command} --endpoint=organizations/1234/locations/us-central1-a/firewallEndpoints/my-endpoint
        """,
  }

  @classmethod
  def Args(cls, parser):
    activation_flags.AddLocationArg(parser)
    api_version = activation_api.GetApiVersion(cls.ReleaseTrack())
    org_spec = activation_flags.OrgEndpointResourceSpec(api_version)
    project_spec = activation_flags.ProjectEndpointResourceSpec(api_version)
    resource_spec = multitype.MultitypeResourceSpec(
        'endpoint', org_spec, project_spec, allow_inactive=True
    )
    presentation_spec = presentation_specs.MultitypeResourcePresentationSpec(
        name='--endpoint',
        concept_spec=resource_spec,
        required=True,
        group_help=(
            'The endpoint to which the verdict change request belongs.'
        ),
    )
    concept_parsers.ConceptParser([presentation_spec]).AddToParser(parser)

  def Run(self, args):
    client = wildfire_verdict_change_requests_api.Client(self.ReleaseTrack())
    endpoint_ref = args.CONCEPTS.endpoint.Parse()
    return client.ListVerdictChangeRequests(
        parent=endpoint_ref.result.RelativeName(),
        limit=args.limit,
        page_size=args.page_size,
    )
